-- ------------ Write CREATE-PROCEDURE-stage scripts -----------

CREATE PROCEDURE orcl.account_pkg$add_brokeraccount(
     IN p_account_id DOUBLE PRECISION, 
     IN p_brokeraccount TEXT, 
     IN p_operator_name TEXT, 
     IN is_primary TEXT DEFAULT 'F')
AS 
$BODY$
DECLARE
    v_account_id DOUBLE PRECISION;
    v_row_count DOUBLE PRECISION;
BEGIN
    SELECT
        COUNT(1)
        INTO STRICT v_row_count
        FROM orcl.brokeraccount_lookup
        WHERE accountid = p_account_id AND brokeraccount = p_brokeraccount;

    IF (v_row_count = 0) THEN
        INSERT INTO orcl.brokeraccount_lookup (accountid, brokeraccount, create_date, create_user, enabled)
        VALUES (p_account_id, p_brokeraccount, aws_oracle_ext.SYSDATE(), p_operator_name, NULL);
        COMMIT;
    END IF;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.account_pkg$insert_invalid_account(
     IN p_broker_account TEXT, 
     IN p_account_name TEXT, 
     IN p_broker_entity TEXT, 
     IN p_client_description TEXT, 
     IN p_file_name TEXT, 
     IN p_client_id DOUBLE PRECISION DEFAULT NULL, 
     IN p_broker_id DOUBLE PRECISION DEFAULT NULL, 
     IN p_account_id DOUBLE PRECISION DEFAULT NULL, 
     IN p_feed_file_id DOUBLE PRECISION DEFAULT NULL, 
     IN p_broker_parent_id DOUBLE PRECISION DEFAULT NULL)
AS 
$BODY$
DECLARE
    v_invalid_id DOUBLE PRECISION;
BEGIN
    v_invalid_id := orcl.account_pkg$pf_insert_invalid_account(p_broker_account, p_account_name, p_broker_entity, p_client_description, p_file_name, p_client_id, p_broker_id, p_account_id, p_feed_file_id, p_broker_parent_id);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.account_pkg$set_account_details(
     IN p_account_id DOUBLE PRECISION, 
     IN p_min_xfer_amount DOUBLE PRECISION, 
     IN p_margin_ccytypeid DOUBLE PRECISION, 
     IN p_transfer_rounding DOUBLE PRECISION)
AS 
$BODY$
BEGIN
    UPDATE orcl.account
    SET min_transfer_amt = p_min_xfer_amount, margin_ccytypeid = p_margin_ccytypeid, transfer_rounding = p_transfer_rounding
        WHERE accountid = p_account_id;
    COMMIT;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.account_pkg$update_brokeraccount(
     IN p_account_id DOUBLE PRECISION, 
     IN p_brokeraccount_old TEXT, 
     IN p_brokeraccount_new TEXT, 
     IN p_operator_name TEXT, 
     IN is_primary TEXT DEFAULT 'F')
AS 
$BODY$
BEGIN
    UPDATE orcl.brokeraccount_lookup
    SET brokeraccount = p_brokeraccount_new, create_user = p_operator_name
        WHERE brokeraccount = p_brokeraccount_old;
    COMMIT;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.aws_oracle_table_mapping(
     IN p_dbname TEXT)
AS 
$BODY$
DECLARE
    v_SQLString CHARACTER VARYING(2000);
    v_data_type CHARACTER VARYING(32);
    v_insert_stmt_number CHARACTER VARYING(512);
    v_stmt CHARACTER VARYING(4000);
    v_suggested_datatype CHARACTER VARYING(64);
    v_suggestion_flg NUMERIC(38);
    v_data_precision NUMERIC(38);
    v_data_scale NUMERIC(38);
    v_sum_val DOUBLE PRECISION;
    v_dbname CHARACTER VARYING(50);
    v_rec RECORD;
BEGIN
    v_dbname := UPPER(p_dbname);
    EXECUTE CONCAT_WS('', 'DELETE FROM orcl.aws_column_mapping     WHERE target_dbname = ''', v_dbname, '''');
    COMMIT;
    v_insert_stmt_number := CONCAT_WS('', 'insert into aws_column_mapping ', ' ( schema_name, table_name, ', ' col_seq, column_name, column_datatype, data_precision, data_scale, is_nullable, ', ' min_val, max_val, sum_val, target_dbname) ');

    LOOP
        /*
        [5591 - Severity CRITICAL - Unable to convert system object SYS.ALL_MVIEWS. Perform a manual conversion., 5578 - Severity CRITICAL - Unable to automatically transform the SELECT statement. Try rewriting the statement.]
        FOR v_rec in (
              select atc.owner table_schema, atc.table_name, atc.column_id as col_seq,
        		atc.column_name, atc.nullable, atc.data_type, atc.data_precision, atc.data_scale
        	  from (
        		select t.owner, t.object_name
        		, row_number() over (partition by t.owner order by t.object_name) table_seq
        			from (
        			select distinct ao.owner, ao.object_name, av.view_name, amv.mview_name
        			, (case when av.view_name is not null then 'VIEW'
        					when amv.mview_name is not null then 'MVIEW'
        				  else 'TABLE' end) obj_type
        			from all_objects  ao
        			left join all_views av on av.view_name = ao.object_name and av.owner = ao.owner
        			left join all_mviews amv on amv.mview_name = ao.object_name and amv.owner = ao.owner
        			where ao.object_type = 'TABLE'
        			and lower(ao.object_name) not like '%tablecounts%'
        			and lower(ao.object_name) not like 'tmp_%'
        			and lower(ao.object_name) not like '%bkp%'
        			and lower(ao.object_name) not like '%$%'
        			and lower(ao.object_name) not like 'aws_column_mapping'
        		) t
        		where t.obj_type = 'TABLE'
        	  ) t2
        	  JOIN  ALL_TAB_COLUMNS atc on atc.owner = t2.owner
        		and atc.table_name = t2.object_name
        		--and atc.owner = sys_context('userenv','current_schema')
        		and atc.OWNER NOT IN ('PUBLIC', 'OUTLN', 'CTXSYS', 'SYSTEM', 'RDSADMIN', 'GSMADMIN_INTERNAL', 'DBSNMP', 'TESTUSER', 'APPQOSSYS', 'XDB', 'SYS')
        	  order by atc.owner, atc.table_name, atc.column_id
          )
        */
        /* dbms_output.put_line( ' 1--- >>> Table: ' || v_rec.table_name || ', Column: ' || v_rec.column_name || ' - ' || systimestamp); */
        v_data_type := v_rec.data_type;

        IF (v_data_type = 'INTEGER' OR v_data_type = 'FLOAT' OR v_data_type = 'DOUBLE' OR v_data_type = 'NUMBER') THEN
            v_SQLString := CONCAT_WS('', 'with rs as (select ', ' nvl(min ( ', v_rec.column_name, ' ),0) min_val ', ', nvl(max ( ', v_rec.column_name, ' ),0) max_val ', ', nvl(sum ( ', v_rec.column_name, ' ),0) sum_val ', '  from ', v_rec.table_schema, '.', v_rec.table_name, ') ');
            v_SQLString := CONCAT_WS('', v_SQLString, 'select ''', v_rec.table_schema, ''',  ''', v_rec.table_name, ''', ', v_rec.col_seq, ', ''', v_rec.column_name, ''',  ''', v_rec.data_type, ''', ', COALESCE(v_rec.data_precision, 0), ', ', COALESCE(v_rec.data_scale, 0), ', ''', v_rec.nullable, '''', ', min_val, max_val, sum_val, ''', v_dbname, ''' from rs ');
            v_stmt := CONCAT_WS('', v_insert_stmt_number, v_SQLString);
            RAISE DEBUG USING MESSAGE = CONCAT_WS('', ' 2--- >>>  v_stmt : ', v_stmt);
            /*
            [5334 - Severity CRITICAL - Unable to convert statements with dynamic SQL statement. Please perform a manual conversion.]
            EXECUTE IMMEDIATE v_stmt
            */
            COMMIT;
        END IF;
    END LOOP;

    FOR v_rec IN
    SELECT
        *
        FROM orcl.aws_column_mapping
        ORDER BY table_name, col_seq
    LOOP
        v_suggestion_flg := 0;
        v_data_precision := NULL;
        v_data_scale := NULL;
        v_suggested_datatype := NULL;
        /* dbms_output.put_line( ' 3--- >>>  v_stmt : ' || v_rec.table_name || ',' || v_rec.column_name); */
        IF (v_rec.column_datatype = 'NUMBER' AND v_rec.data_precision = 0 AND v_rec.data_scale = 0) THEN
            IF (v_rec.min_val >= - 2147483648 AND v_rec.max_val <= 2147483647) THEN
                IF (v_dbname = 'POSTGRESQL') THEN
                    v_suggested_datatype := 'INTEGER';
                ELSIF (v_dbname = 'MYSQL') THEN
                    v_suggested_datatype := 'INT UNSIGNED';
                END IF;
            ELSE
                IF (v_dbname = 'POSTGRESQL') THEN
                    v_suggested_datatype := 'BIGINT';
                ELSIF (v_dbname = 'MYSQL') THEN
                    v_suggested_datatype := 'BIGINT UNSIGNED';
                END IF;
                v_suggestion_flg := 1;
            END IF;
            v_sum_val := v_rec.sum_val;

            IF (aws_oracle_ext.INSTR(v_sum_val::TEXT, '.', 1) > 0) THEN
                IF (v_dbname = 'POSTGRESQL') THEN
                    v_suggested_datatype := 'NUMBER';
                ELSIF (v_dbname = 'MYSQL') THEN
                    v_suggested_datatype := 'DECIMAL';
                END IF;
                v_data_precision := v_rec.max_val;
                v_data_scale := LENGTH(aws_oracle_ext.substr(v_sum_val::TEXT, aws_oracle_ext.INSTR(v_sum_val::TEXT, '.', 1) + 1, LENGTH(v_sum_val::TEXT)));
            END IF;
        ELSIF (v_rec.column_datatype = 'NUMBER' AND (v_rec.data_precision > 0 OR v_rec.data_scale > 0)) THEN
            IF (v_dbname = 'POSTGRESQL') THEN
                v_suggested_datatype := 'NUMBER';
            ELSIF (v_dbname = 'MYSQL') THEN
                v_suggested_datatype := 'DECIMAL';
            END IF;
        ELSE
            v_suggested_datatype := 'DOUBLE PRECISION';
        END IF;

        IF (v_suggested_datatype = 'NUMBER') OR (v_suggested_datatype = 'DECIMAL') THEN
            UPDATE orcl.aws_column_mapping
            SET suggested_datatype = v_suggested_datatype, suggestion_flg = v_suggestion_flg, suggested_data_precision = v_data_precision, suggested_data_scale = v_data_scale
                WHERE target_dbname = v_dbname AND table_name = v_rec.table_name AND col_seq = v_rec.col_seq AND column_name = v_rec.column_name;
        ELSE
            UPDATE orcl.aws_column_mapping
            SET suggested_datatype = v_suggested_datatype, suggestion_flg = v_suggestion_flg, suggested_data_precision = v_data_precision, suggested_data_scale = v_data_scale
                WHERE target_dbname = v_dbname AND table_name = v_rec.table_name AND col_seq = v_rec.col_seq AND column_name = v_rec.column_name;
        END IF;
        COMMIT;
    END LOOP;
/* dbms_output.put_line( 'Table Column Mapping completed... '); */
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.dl_load$refresh_all_groups()
AS 
$BODY$
DECLARE
    job DOUBLE PRECISION;
    c CURSOR FOR
    SELECT
        group_name, orcl.dl_load$get_inactive_pid(active_partition) AS inactive_id, error_message
        FROM orcl.partition_switch
        WHERE ((start_timestamp IS NULL AND finish_timestamp IS NULL) OR (start_timestamp IS NOT NULL AND finish_timestamp IS NOT NULL AND finish_timestamp + (aws_oracle_ext.TO_NUMBER(refresh_interval) / (24 * 60)::NUMERIC || ' days')::INTERVAL <= aws_oracle_ext.SYSDATE())) AND refresh_interval IS NOT NULL;
/* AND   error_message IS NULL; */
BEGIN
    FOR rgrp IN c LOOP
        IF COALESCE(rGrp.error_message, 'X') <> 'INTERNAL' THEN
            CALL orcl.dl_load$submit_refresh_group(p_group_name => rGrp.group_name, p_refresh_pid => rGrp.inactive_id);
        END IF;
    END LOOP;
    EXCEPTION
        WHEN others THEN
            RAISE;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.dl_load$refresh_group(
     IN p_group_name TEXT, 
     IN p_inactive_pid INTEGER)
AS 
$BODY$
DECLARE
    v_error_message CHARACTER VARYING(1000);
    b_success BOOLEAN;
    ctab CURSOR FOR
    SELECT
        table_name, source_name
        FROM orcl.partition_group
        WHERE group_name = p_group_name;
BEGIN
    PERFORM ORCL.DL_UTILITIES$Init();
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_OUTPUT.ENABLE(INTEGER) function. Use suitable function or create user defined function.]
    DBMS_OUTPUT.ENABLE(1000000)
    */
    v_error_message := NULL;
    v_error_message := 'BEFORE dl_utilities.update_partition_switch';
    CALL orcl.dl_utilities$update_partition_switch(p_stage => aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_initial_load'), p_group_name => p_group_name);
    b_success := TRUE;

    FOR rtab IN ctab LOOP
        IF b_success THEN
            b_success := orcl.dl_load$refresh_table(p_group_name := p_group_name, p_table_name := rTab.table_name, p_inactive_pid := p_inactive_pid, p_source_name := rTab.source_name);
        END IF;
    END LOOP;

    IF b_success THEN
        CALL orcl.dl_utilities$update_partition_switch(p_stage => aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_end_load'), p_group_name => p_group_name);
    END IF;
    EXCEPTION
        WHEN others THEN
            v_error_message := CONCAT_WS('', SQLERRM, ' ', v_error_message);
            /* DBMS_OUTPUT.PUT_LINE ('Refresh Group (EXCEPTION) - '||p_group_name||' : '||v_error_message); */
            CALL orcl.dl_utilities$update_partition_switch(p_stage => aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_error_load'), p_group_name => p_group_name, p_error_message => v_error_message);
            RAISE;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.dl_load$submit_refresh_group(
     IN p_group_name TEXT, 
     IN p_refresh_pid INTEGER)
AS 
$BODY$
DECLARE
    /*
    [5571 - Severity CRITICAL - Unable to convert AutonomousTransaction pragma. Check converted code for perfomance issues and rewrite it manually, if necessary.]
    PRAGMA AUTONOMOUS_TRANSACTION
    */
    v_job INTEGER;
BEGIN
    SELECT
        *
        FROM aws_oracle_ext.dbms_job$submit(what := CONCAT_WS('', 'BEGIN DL_Load.refresh_group(''', p_group_name, ''',', p_refresh_pid, '); END;'))
        INTO v_job;
    COMMIT;
    EXCEPTION
        WHEN others THEN
            RAISE;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.dl_utilities$update_partition_switch(
     IN p_stage TEXT, 
     IN p_group_name TEXT, 
     IN p_error_message TEXT DEFAULT NULL)
AS 
$BODY$
/*
[5571 - Severity CRITICAL - Unable to convert AutonomousTransaction pragma. Check converted code for perfomance issues and rewrite it manually, if necessary.]
PRAGMA AUTONOMOUS_TRANSACTION
*/
BEGIN
    PERFORM ORCL.DL_UTILITIES$Init();

    IF p_stage = aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_initial_load') THEN
        UPDATE orcl.partition_switch
        SET start_timestamp = aws_oracle_ext.SYSDATE(), error_message = 'INTERNAL'
            WHERE group_name = p_group_name;
    ELSIF p_stage = aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_end_load') THEN
        UPDATE orcl.partition_switch
        SET finish_timestamp = aws_oracle_ext.SYSDATE(), error_message = NULL, active_partition = orcl.dl_load$get_inactive_pid(active_partition)
            WHERE group_name = p_group_name;
    ELSIF p_stage = aws_oracle_ext.get_package_variable('ORCL', 'DL_UTILITIES', 'c_error_load') THEN
        UPDATE orcl.partition_switch
        SET error_message = p_error_message
            WHERE group_name = p_group_name;
    END IF;
    COMMIT;
    EXCEPTION
        WHEN others THEN
            RAISE;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.load_dim_mandate_detail()
AS 
$BODY$
DECLARE
    v_territories CHARACTER VARYING(4000);
    combo_cur CURSOR FOR
    SELECT DISTINCT
        target_territory_code, country_codes
        FROM stg_distinct_mandate_combos;
    cur_ww_minus CURSOR FOR
    SELECT
        *
        FROM stg_dim_mandate_detail
        WHERE target_territory_code = 'WW-'
        ORDER BY mandate_dim_id;
    cur_r CURSOR FOR
    SELECT
        *
        FROM stg_dim_mandate_detail
        WHERE target_territory_code = 'R'
        ORDER BY mandate_dim_id;
    cur_ww CURSOR FOR
    SELECT
        *
        FROM stg_dim_mandate_detail
        WHERE target_territory_code = 'WW'
        ORDER BY mandate_dim_id;
BEGIN
    EXECUTE 'TRUNCATE TABLE stg_dim_mandate_detail';
    INSERT INTO stg_dim_mandate_detail
    SELECT
        mandate_dim_id, SUBSTRING(target_territories FROM 1 FOR 3000) AS target_territories, aws_oracle_ext.substr(REPLACE(SUBSTRING(target_territories FROM 1 FOR 3000), ' ', ''), aws_oracle_ext.INSTR(REPLACE(SUBSTRING(target_territories FROM 1 FOR 3000), ' ', ''), '"t":"') + 5, aws_oracle_ext.INSTR(REPLACE(SUBSTRING(target_territories FROM 1 FOR 3000), ' ', ''), '","s":"') - 7) AS target_territory_code, REPLACE(aws_oracle_ext.substr(REPLACE(SUBSTRING(target_territories FROM 1 FOR 3000), ' ', ''), aws_oracle_ext.INSTR(REPLACE(SUBSTRING(target_territories FROM 1 FOR 3000), ' ', ''), '"s":"') + 5), '"}', '') AS country_codes, REPLACE(REPLACE(conditional_aspects, '{', ''), '}', '') AS conditional_aspects, REPLACE(REPLACE(performance_types, '{', ''), '}', '') AS performance_types, REPLACE(REPLACE(channels, '{', ''), '}', '') AS channels, aws_oracle_ext.SYSDATE() AS dw_created_date
        FROM dim_mandate AS a
        WHERE NOT EXISTS (SELECT
            1
            FROM dim_mandate_detail AS b
            WHERE a.mandate_dim_id = b.mandate_dim_id);
    COMMIT;
    EXECUTE 'TRUNCATE TABLE stg_dim_mandate_detail_country';
    INSERT INTO stg_dim_mandate_detail_country
    SELECT
        mandate_dim_id, target_territories, target_territory_code, country_codes, TRIM(aws_oracle_ext.regexp_substr((country_codes::TEXT)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels.column_value::NUMERIC)::INTEGER)::NUMERIC)) AS country_code, COALESCE(LENGTH(aws_oracle_ext.regexp_replace((country_codes::TEXT)::TEXT, ('[^,]+')::TEXT)), 0) + 1 AS count, aws_oracle_ext.SYSDATE()
        FROM stg_dim_mandate_detail AS r,
        /* Transformer error occurred */;
    COMMIT;
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_STATS.GATHER_TABLE_STATS(VARCHAR2,VARCHAR2,VARCHAR2,NUMBER,BOOLEAN,VARCHAR2,NUMBER,VARCHAR2,BOOLEAN,VARCHAR2,VARCHAR2,VARCHAR2,BOOLEAN,VARCHAR2,BOOLEAN,DBMS_STATS.CCONTEXT,VARCHAR2) function. Use suitable function or create user defined function.]
    dbms_stats.gather_table_stats('SX_EDW_DIM','STG_DIM_MANDATE_DETAIL_COUNTRY',cascade=>TRUE,no_invalidate=>FALSE)
    */
    EXECUTE 'TRUNCATE TABLE stg_distinct_mandate_combos';
    INSERT INTO stg_distinct_mandate_combos
    SELECT DISTINCT
        target_territory_code, country_codes, country_code
        FROM stg_dim_mandate_detail_country AS c
        WHERE target_territory_code IN ('WW-', 'R');
    COMMIT;
    EXECUTE 'TRUNCATE TABLE stg_distinct_mandate_other';

    FOR rec IN combo_cur LOOP
        /* This looks at distinct combinations of mandate type and country codes and generates the distinct list of opposite country codes for use in WW- and R */
        INSERT INTO stg_distinct_mandate_other
        SELECT
            rec.target_territory_code, rec.country_codes, STRING_AGG(b.country_code, ','::TEXT ORDER BY b.country_code)
            FROM sx_edw_stg.ref_country_codes AS b
            LEFT OUTER JOIN stg_distinct_mandate_combos AS a
                ON a.country_code = b.country_code AND target_territory_code = rec.target_territory_code AND country_codes = rec.country_codes
            WHERE a.country_code IS NULL;
        COMMIT;
    END LOOP;

    FOR rec IN cur_ww_minus LOOP
        SELECT
            other_country_codes
            INTO STRICT v_territories
            FROM stg_distinct_mandate_other
            WHERE target_territory_code = rec.target_territory_code AND country_codes = rec.country_codes;
        INSERT INTO dim_mandate_detail
        VALUES (rec.mandate_dim_id, rec.target_territories, rec.target_territory_code, v_territories, rec.country_codes,
        CASE
            WHEN rec.performance_types IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_performance_types)
            ELSE rec.performance_types
        END,
        CASE
            WHEN rec.channels IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_channels)
            ELSE rec.channels
        END,
        CASE
            WHEN rec.conditional_aspects IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_conditional_aspects)
            ELSE rec.conditional_aspects
        END, aws_oracle_ext.SYSDATE(), NULL, NULL);
        COMMIT;
    END LOOP;

    FOR rec IN cur_r LOOP
        SELECT
            other_country_codes
            INTO STRICT v_territories
            FROM stg_distinct_mandate_other
            WHERE target_territory_code = rec.target_territory_code AND country_codes = rec.country_codes;
        INSERT INTO dim_mandate_detail
        VALUES (rec.mandate_dim_id, rec.target_territories, rec.target_territory_code, rec.country_codes, v_territories,
        CASE
            WHEN rec.performance_types IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_performance_types)
            ELSE rec.performance_types
        END,
        CASE
            WHEN rec.channels IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_channels)
            ELSE rec.channels
        END,
        CASE
            WHEN rec.conditional_aspects IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_conditional_aspects)
            ELSE rec.conditional_aspects
        END, aws_oracle_ext.SYSDATE(), NULL, NULL);
        COMMIT;
    END LOOP;
    /* For WW mandates all countries are included, so this only needs to be done once */
    SELECT
        STRING_AGG(country_code, ','::TEXT ORDER BY country_code)
        INTO STRICT v_territories
        FROM sx_edw_stg.ref_country_codes;

    FOR rec IN cur_ww LOOP
        INSERT INTO dim_mandate_detail
        VALUES (rec.mandate_dim_id, rec.target_territories, rec.target_territory_code, v_territories, NULL,
        CASE
            WHEN rec.performance_types IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_performance_types)
            ELSE rec.performance_types
        END,
        CASE
            WHEN rec.channels IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_channels)
            ELSE rec.channels
        END,
        CASE
            WHEN rec.conditional_aspects IS NULL THEN (SELECT
                STRING_AGG(code, ','::TEXT ORDER BY code)
                FROM sx_edw_stg.ref_conditional_aspects)
            ELSE rec.conditional_aspects
        END, aws_oracle_ext.SYSDATE(), NULL, NULL);
        COMMIT;
    END LOOP;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.mviews_refresh_pkg$refresh_eln_experiments()
AS 
$BODY$
/* ----------------------------------------------------------------------------- */
/* Name:refresh_eln_experiments */
/* Description: Procedure to load data from ELN tables into */
/* eln_sample_result_temp. If successful, copy data into eln_sample_result_mv */
/* */
/* Output: None. */
/* ---------------------------------------------- */
DECLARE
    v_begin_time DOUBLE PRECISION
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    := DBMS_UTILITY.get_time
    */;
    v_date TIMESTAMP(0) WITHOUT TIME ZONE := aws_oracle_ext.SYSDATE();
    v_message CHARACTER VARYING(3000);
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'refresh_eln_experiments';
/* gc_crlf constant varchar2(2) := chr(13) || chr(10); */
/* gc_pkg_name constant varchar2(30) := 'mviews_refresh_pkg'; */
BEGIN
    PERFORM ORCL.MVIEWS_REFRESH_PKG$Init();
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'S'::TEXT, NULL);
    /* truncate the temp table for ELN result */
    EXECUTE 'TRUNCATE TABLE orcl.eln_sample_result_temp';
    /* REFRESH the temp table for ELN result */
    /*
    [9996 - Severity CRITICAL - Transformer error occurred. Please submit report to developers.]
    INSERT INTO ELN_SAMPLE_RESULT_TEMP
             SELECT *
               FROM (WITH sample_view
                          AS (SELECT M.NAME AS SampleName,
                                     U.FULLNAME AS CREATOR,
                                     S.CONTAINERID AS DOCUMENTID,
                                     S.BATCHNUMBER,
                                     S.SAMPLEID,
                                     s.MATERIALTYPE AS sample_desc,
                                     S.SAMPLETYPE,
                                     S.ISBMSAPI,
                                     S.BLEND,
                                     S.SAPNUMBER,
                                     S.BMSBMTNUMBER,
                                     S.EHSBAND,
                                     S.EXCIPIENTNAME,
                                     S.GRANULATIONTYPE,
                                     S.MATERIALTYPE,
                                     S.OTHERID,
                                     S.NOTE,
                                     V.AUTONAME AS elnnumber,
                                     s.creationdate
                                FROM P_MPDBSampleinformation_obj S,
                                     P_Material_OBJ M,
                                     VAULTOBJECT_OBJ V,
                                     G_USER U
                               WHERE     S.VAULTID = M.VAULTID
                                     AND S.OWNERID = M.OWNERID
                                     AND S.CONTAINERID = M.CONTAINERID
                                     AND S.CONTAINERID = V.VAULTID
                                     AND V.CREATOR = U.GUID)
                     SELECT *
                       FROM (SELECT 70077777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    -- 'No' AS mergedSample,
                                    --    experiment info
                                    'Disintegration' AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Disintegration',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Disintegration',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    --p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (TABLET1WT), ' ')
                                       "978646001656847232",
                                    NVL (TO_CHAR (TABLET2WT), ' ')
                                       "452881439647127104",
                                    NVL (TO_CHAR (TABLET3WT), ' ')
                                       "305352551906772160",
                                    NVL (TO_CHAR (TABLET4WT), ' ')
                                       "615125137535039360",
                                    NVL (TO_CHAR (TABLET5WT), ' ')
                                       "122607922234973520",
                                    NVL (TO_CHAR (TABLET6WT), ' ')
                                       "890819932818838656",
                                    NVL (TO_CHAR (DISINTEGTIMEFIRSTTAB), ' ')
                                       "38443140116054984",
                                    NVL (TO_CHAR (DISINTEGRATIONTIMELASTTAB),
                                         ' ')
                                       "127153989501293200",
                                    NVL (TO_CHAR (RH), ' ') "411212320281695424",
                                    NVL (TO_CHAR (TEMP), ' ')
                                       "677130996910768640",
                                    NVL (TO_CHAR (DISINTEGRATIONTIMETAB2), ' ')
                                       "998215661964566400",
                                    NVL (TO_CHAR (DISINTEGRATIONTIMETAB3), ' ')
                                       "969308469616769280",
                                    NVL (TO_CHAR (DISINTEGRATIONTIMETAB4), ' ')
                                       "160986850276606432",
                                    NVL (TO_CHAR (DISINTEGRATIONTIMETAB5), ' ')
                                       "905342573522622976",
                                    NVL (TO_CHAR (DISTIMEUNITS1), ' ')
                                       "577525268489691392",
                                    NVL (TO_CHAR (DISTIMEUNITS2), ' ')
                                       "174583875351247968",
                                    NVL (TO_CHAR (DISTIMEUNITS3), ' ')
                                       "678470612553983616",
                                    NVL (TO_CHAR (DISTIMEUNITS4), ' ')
                                       "644676543038518016",
                                    NVL (TO_CHAR (DISTIMEUNITS5), ' ')
                                       "179214827816068800",
                                    NVL (TO_CHAR (DISTIMEUNITS6), ' ')
                                       "121822873237952496"
                               FROM P_MPDBDISINTEGRATION_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("978646001656847232",
                                                                        "452881439647127104",
                                                                        "305352551906772160",
                                                                        "615125137535039360",
                                                                        "122607922234973520",
                                                                        "890819932818838656",
                                                                        "38443140116054984",
                                                                        "127153989501293200",
                                                                        "411212320281695424",
                                                                        "677130996910768640",
                                                                        "998215661964566400",
                                                                        "969308469616769280",
                                                                        "160986850276606432",
                                                                        "905342573522622976",
                                                                        "577525268489691392",
                                                                        "174583875351247968",
                                                                        "678470612553983616",
                                                                        "644676543038518016",
                                                                        "179214827816068800",
                                                                        "121822873237952496"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70177777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    --  'No' AS mergedSample,
                                    --    experiment info
                                    'FT4 Powder Rheometry' AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'FT4 Powder Rheometry',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'FT4 Powder Rheometry',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (prescreenconducted), ' ') "Pre",
                                    NVL (TO_CHAR (temperature), ' ')
                                       "Temperature",
                                    NVL (TO_CHAR (relativehumidity), ' ')
                                       "Humidity",
                                    NVL (TO_CHAR (bd), ' ') "BD",
                                    NVL (TO_CHAR (bfe), ' ') "BFE",
                                    NVL (TO_CHAR (si), ' ') "SI",
                                    NVL (TO_CHAR (fri), ' ') "FRI",
                                    NVL (TO_CHAR (se), ' ') "SE",
                                    NVL (TO_CHAR (instrumentused), ' ')
                                       "Instrument",
                                    NVL (TO_CHAR (samplevesselused), ' ')
                                       "Samplevessel"
                               FROM p_powderrheometry_obj P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.VAULTID = S.VAULTID
                                    AND P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("Pre",
                                                                        "Temperature",
                                                                        "Humidity",
                                                                        "BD",
                                                                        "BFE",
                                                                        "SI",
                                                                        "FRI",
                                                                        "SE",
                                                                        "Instrument",
                                                                        "Samplevessel"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT   70277777
                                    + RANK () OVER (ORDER BY samplename, creator)
                                       AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    --  'No' AS mergedSample,
                                    --    experiment info
                                    'Granule Friability' AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Granule Friability',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Granule Friability',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (TEMPERATURE), ' ')
                                       "436033068444116480",
                                    NVL (TO_CHAR (RH), ' ') "349617024225448384",
                                    -- NVL (to_char(REPLICATENAME2), ' ') REPLICATENAME2,
                                    -- NVL (to_char(FRIABILITY), ' ') FRIABILITY
                                    CASE
                                       WHEN LOWER (replicatename2) =
                                               'replicate 1'
                                       THEN
                                          TO_CHAR (FRIABILITY)
                                    END
                                       "124748585160566640",
                                    CASE
                                       WHEN LOWER (replicatename2) =
                                               'replicate 2'
                                       THEN
                                          TO_CHAR (FRIABILITY)
                                    END
                                       "153995258333783456"
                               FROM P_MPDBGRANULEFRIABILITY1_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("436033068444116480",
                                                                        "349617024225448384",
                                                                        --REPLICATENAME2,
                                                                        -- FRIABILITY
                                                                        "124748585160566640",
                                                                        "153995258333783456"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70377777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    --'No' AS mergedSample,
                                    --    experiment info
                                    'BET Specific Surface Area'
                                       AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'BET Specific Surface Area',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'BET Specific Surface Area',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (MULTIPOINTSURFACEAREAMEAN),
                                         ' ')
                                       "SSAm",
                                    NVL (TO_CHAR (SINGLEPOINTSURFACEAREAMEAN),
                                         ' ')
                                       "SSAs",
                                    NVL (TO_CHAR (AVGPOREVOLMEAN), ' ') "PV",
                                    NVL (TO_CHAR (AVGPOREWIDTHMEAN), ' ') "PS"
                               FROM P_BETSURFACEAREA_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("SSAm",
                                                                        "SSAs",
                                                                        "PV",
                                                                        "PS"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70477777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    -- 'No' AS mergedSample,
                                    --    experiment info
                                    'Surface Energy Analysis' AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Surface Energy Analysis',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Surface Energy Analysis',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (DISPERSIVESURFACEENERGY5), ' ')
                                       "DSE",
                                    NVL (TO_CHAR (FREEENERGYETHYLACETATE), ' ')
                                       "dGEthylacetate",
                                    NVL (TO_CHAR (FREEENERGYCHLOROFORM), ' ')
                                       "dGChloroform",
                                    NVL (TO_CHAR (SPECIFICSURFACEENERGY), ' ')
                                       "SSE",
                                    NVL (TO_CHAR (WORKOFCOHESION), ' ')
                                       "WCohesion"
                               FROM P_SURFACEENERGYANALYZER_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("DSE",
                                                                        "dGEthylacetate",
                                                                        "dGChloroform",
                                                                        "SSE",
                                                                        "WCohesion"))
                     UNION ALL
                     SELECT * FROM tablet_sample_result_vw
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70677777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    --'No' AS mergedSample,
                                    --    experiment info
                                    'BULK and Tap Density | BPRD-DM0071'
                                       AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'BULK and Tap Density | BPRD-DM0071',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'BULK and Tap Density | BPRD-DM0071',
                                       'P')
                                       AS experimentCode,
                                    --  Results    Info
                                    -- p.calvalue,
                                    --p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (bulkdensity1), ' ')
                                       "bulkDensity1",
                                    NVL (TO_CHAR (bulkdensity2), ' ')
                                       "bulkDensity2",
                                    NVL (TO_CHAR (tapdensity1), ' ')
                                       "tapDensity1",
                                    NVL (TO_CHAR (tapdensity2), ' ')
                                       "tapDensity2"
                               FROM p_mpdbbulktapdensity_obj p,
                                    p_mpdbsampleinfo_obj s,
                                    sample_view sv
                              WHERE     p.vaultid = s.vaultid
                                    AND p.ownerid = s.ownerid
                                    AND p.containerid = s.containerid
                                    AND p.parentid = s.parentid
                                    AND s.containerid = sv.documentid
                                    AND s.sampleid = sv.samplename) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("bulkDensity1",
                                                                        "bulkDensity2",
                                                                        "tapDensity1",
                                                                        "tapDensity2"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70777777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    'Helium Pycnometry (True density)'
                                       AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Helium Pycnometry (True density)',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Helium Pycnometry (True density)',
                                       'E')
                                       AS experimentCode,
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    --'No' AS mergedSample,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (VOLUMEMEAN), ' ')
                                       "797303460988053888",
                                    NVL (TO_CHAR (TRUEDENSITYMEAN), ' ')
                                       "473745136580581376"
                               FROM P_ACCUPYC_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("797303460988053888",
                                                                        "473745136580581376"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70877777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    sv.SAMPLE_DESC,
                                    --'No' AS mergedSample,
                                    'FT4| Shear Cell' AS experimentTypeName,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'FT4| Shear Cell',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'FT4| Shear Cell',
                                       'E')
                                       AS experimentCode,
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    --NVL (PRESCREENCONDUCTED, ' ') "Pre",
                                    NVL (TEMPERATURE, ' ') "Temperature",
                                    NVL (RELATIVEHUMIDITY, ' ') "Humidity",
                                    NVL (UYS, ' ') "UYS",
                                    NVL (MPS, ' ') "MPS",
                                    NVL (C, ' ') "C",
                                    NVL (FFC, ' ') "ffc",
                                    NVL (AIF, ' ') "AIF",
                                    NVL (AIFE, ' ') "AIFE",
                                    NVL (AIFS, ' ') "AIFSS",
                                    NVL (TO_CHAR (PRECONSOLIDATIONFORCE), ' ')
                                       "Pre",
                                    NVL (TO_CHAR (INSTRUMENTUSED), ' ')
                                       "Instrument",
                                    NVL (TO_CHAR (SAMPLEVESSELUSED), ' ')
                                       "Samplevessel",
                                    NVL (TO_CHAR (MCS), ' ') "MCS",
                                    NVL (TO_CHAR (REIP), ' ') "ReIp",
                                    NVL (TO_CHAR (COMPRESSEDDENSITY), ' ')
                                       "9372308982553812",
                                    NVL (TO_CHAR (WALLFRICTIONANGLE), ' ')
                                       "365385219210016128"
                               FROM P_SHEARCELL_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV
                              WHERE     P.VAULTID = S.VAULTID
                                    AND P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    --AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ( --"Pre",
                                                                         "Temperature",
                                                                        "Humidity",
                                                                        "UYS",
                                                                        "MPS",
                                                                        "C",
                                                                        "ffc",
                                                                        "AIF",
                                                                        "AIFE",
                                                                        "AIFSS",
                                                                        "Pre",
                                                                        "Instrument",
                                                                        "Samplevessel",
                                                                        "MCS",
                                                                        "ReIp",
                                                                        "9372308982553812",
                                                                        "365385219210016128"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 70977777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    -- 'No' AS mergedSample,
                                    --  'No' AS mergedSample,
                                    -- Experiment info
                                    'Mercury Porosimetry' AS experimentTypeName,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Mercury Porosimetry',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Mercury Porosimetry',
                                       'E')
                                       AS experimentCode,
                                    --Result info
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (INTRUSIONVOLUME), ' ')
                                       "575286065151181888",
                                    NVL (TO_CHAR (TOTALPOREAREA), ' ')
                                       "784867229155296768",
                                    NVL (TO_CHAR (MEDIANPOREDIAMETER), ' ')
                                       "818157221940903296",
                                    NVL (TO_CHAR (MEDIAMPOREDIAMETER2), ' ')
                                       "962104181416079232",
                                    NVL (TO_CHAR (AVERAGEPOREDIAMETER), ' ')
                                       "933005226226004224",
                                    NVL (TO_CHAR (HYSTERESIS), ' ')
                                       "954386580744698496",
                                    NVL (TO_CHAR (POROSITY), ' ')
                                       "298384784677282112",
                                    NVL (TO_CHAR (BULKDENSITY), ' ')
                                       "553902644892011328",
                                    NVL (TO_CHAR (SKELETALDENSITY), ' ')
                                       "251180533336278208",
                                    --Version 2
                                    NVL (TO_CHAR (LOWRANGEINTVOL), ' ')
                                       "443274377957861376",
                                    NVL (TO_CHAR (HIGHRANGEINTVOL), ' ')
                                       "808400615490230784"
                               FROM P_MERCURYPOROSITY_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    Sample_View SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("575286065151181888",
                                                                        "784867229155296768",
                                                                        "818157221940903296",
                                                                        "962104181416079232",
                                                                        "933005226226004224",
                                                                        "954386580744698496",
                                                                        "298384784677282112",
                                                                        "553902644892011328",
                                                                        "251180533336278208",
                                                                        --Version 2
                                                                        "443274377957861376",
                                                                        "808400615490230784"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 71077777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    'Angle of Repose' AS experimentTypeName,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Angle of Repose',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Angle of Repose',
                                       'E')
                                       AS experimentCode,
                                    --  p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (ANGOFREP), ' ')
                                       "599815651346909568",
                                    NVL (TO_CHAR (TEMP_VALUE), ' ')
                                       "253427800679835936",
                                    NVL (TO_CHAR (HUMIDITY1_VALUE), ' ')
                                       "330170029057852736"
                               FROM P_ANGLEOFREPOSE_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    Sample_View SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("599815651346909568",
                                                                        "253427800679835936",
                                                                        "330170029057852736"))
    
                     UNION ALL
                     SELECT *
                       FROM (SELECT 71777777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    'Stylcam (API only)' AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Stylcam (API only)',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Stylcam (API only)',
                                       'E')
                                       AS experimentCode,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (P.ACTUALDWELLTIME), ' ')
                                       "562186116094767744",
                                    NVL (TO_CHAR (P.COMPACTIONPRESSURE), ' ')
                                       "553140516486525952",
                                    NVL (TO_CHAR (P.EJECTIONFORCE), ' ')
                                       "141940569737696416",
                                    NVL (TO_CHAR (P.POROSITY), ' ')
                                       "644551062814551936",
                                    NVL (TO_CHAR (P.RECOVERY), ' ')
                                       "373735615120442816",
                                    NVL (TO_CHAR (P.SOLIDFRACTION), ' ')
                                       "495234822262157760",
                                    NVL (TO_CHAR (P.TENSILESTRENGTH), ' ')
                                       "799438514308295552",
                                    NVL (TO_CHAR (P.THEORETICALDWELLTIME), ' ')
                                       "161180715077679488",
                                    NVL (TO_CHAR (P.YIELDPRESSURE), ' ')
                                       "13085459232656582"
                               FROM P_MPDBSTYLCAMAPI_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    Sample_View SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("562186116094767744",
                                                                        "553140516486525952",
                                                                        "141940569737696416",
                                                                        "644551062814551936",
                                                                        "373735615120442816",
                                                                        "495234822262157760",
                                                                        "799438514308295552",
                                                                        "161180715077679488",
                                                                        "13085459232656582"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 71877777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    'Scanning Electron Microscopy'
                                       AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Scanning Electron Microscopy',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Scanning Electron Microscopy',
                                       'E')
                                       AS experimentCode,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (P.INSTRUMENTUSED), ' ')
                                       "890337315468562560",
                                    NVL (TO_CHAR (P.OBSERVATIONSIZE), ' ')
                                       "324636739508709312",
                                    NVL (TO_CHAR (P.PARTICLEHABIT), ' ')
                                       "769526722898456192"
                               FROM P_MPDBELECTRONMICROSCOPY_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    Sample_View SV
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName) UNPIVOT (calvalue
                                                                    FOR calname
                                                                    IN  ("890337315468562560",
                                                                        "324636739508709312",
                                                                        "769526722898456192"))
    
            ---- ADD OR UPDATE NEW TABLES FOR V2.2-----------------------------------------------------------------
         ---- Experiment Morphologi Particle Characterization -----------------------
            UNION ALL
            SELECT *
      FROM (SELECT 70577777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'Morphologi Particle Characterization' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('Morphologi Particle Characterization',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('Morphologi Particle Characterization', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                    NVL (TO_CHAR (GPSDV10), ' ') "Dv10",
                    NVL (TO_CHAR (GPSDV50), ' ') "Dv50",
                    NVL (TO_CHAR (GPSDV90), ' ') "Dv90",
                    NVL (TO_CHAR (APSDN10), ' ') "Dn10",
                    NVL (TO_CHAR (APSDN50), ' ') "Dn50",
                    NVL (TO_CHAR (APSDN90), ' ') "Dn90",
                    NVL (TO_CHAR (APEEN10), ' ') "En10",
                    NVL (TO_CHAR (APEEN50), ' ') "En50",
                    NVL (TO_CHAR (APEEN90), ' ') "En90",
                    NVL (TO_CHAR (GPEEV10), ' ') "Ev10",
                    NVL (TO_CHAR (GPEEV50), ' ') "Ev50",
                    NVL (TO_CHAR (GPEEV90), ' ') "Ev90",
                    NVL (TO_CHAR (Agglomerated), ' ') "Agglomerated",
                    NVL (TO_CHAR (NOOFPARTICLES), ' ') "NoOfParticles"
    
    
    
                 FROM P_PARTICLESIZE_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  (
                                        "Dv10",
                                        "Dv50",
                                        "Dv90",
                                        "Dn10",
                                        "Dn50",
                                        "Dn90",
                                        "En10",
                                        "En50",
                                        "En90",
                                        "Ev10",
                                        "Ev50",
                                        "Ev90",
                                        "Agglomerated",
                                        "NoOfParticles"
                                   ))
    
      ---- Morphologi G3 light intensity, length and width -----------------------
        UNION ALL
    
        SELECT *
            FROM (SELECT 71977777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'Morphologi Shape Data' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('Morphologi Shape Data',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('Morphologi Shape Data', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                    --- NEW FIELDS ADD FOR V2.2
                   NVL (TO_CHAR (ALIn10), ' ') "970294381840932736",
                   NVL (TO_CHAR (ALIn50), ' ') "541842245986741120",
                   NVL (TO_CHAR (ALIn90), ' ') "463410092562777536",
    
                   NVL (TO_CHAR (GLIv10), ' ') "619555141908416256",
                   NVL (TO_CHAR (GLIv50), ' ') "771597012611030528",
                   NVL (TO_CHAR (GLIv90), ' ') "115140625812470304",
    
                   NVL (TO_CHAR (ALLn10), ' ') "213872345613662080",
                   NVL (TO_CHAR (ALLn50), ' ') "204462767007278624",
                   NVL (TO_CHAR (ALLn90), ' ') "109698449653128464",
    
                   NVL (TO_CHAR (GLLv10), ' ') "265131807602645312",
                   NVL (TO_CHAR (GLLv50), ' ') "747257394167365248",
                   NVL (TO_CHAR (GLLv90), ' ') "607684072381928960",
    
                   NVL (TO_CHAR (AWWn10), ' ') "304308300685363328",
                   NVL (TO_CHAR (AWWn50), ' ') "808333498716850688",
                   NVL (TO_CHAR (AWWn90), ' ') "675882675858280192",
    
                   NVL (TO_CHAR (GWWv10), ' ') "233782418941640992",
                   NVL (TO_CHAR (GWWv50), ' ') "358587701917589568",
                   NVL (TO_CHAR (GWWv90), ' ') "395198628472317824"
    
    
                 FROM P_PARTICLESIZE_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  (
                                        "970294381840932736",
                                        "541842245986741120",
                                        "463410092562777536",
                                        "619555141908416256",
                                        "771597012611030528",
                                        "115140625812470304",
                                        "213872345613662080",
                                        "204462767007278624",
                                        "109698449653128464",
                                        "265131807602645312",
                                        "747257394167365248",
                                        "607684072381928960",
                                        "304308300685363328",
                                        "808333498716850688",
                                        "675882675858280192",
                                        "233782418941640992",
                                        "358587701917589568",
                                        "395198628472317824"
    
                                   ))
    
    ---- Experiment BBRC_Crystallization -----------------------
            UNION ALL
            SELECT *
            FROM (SELECT 72077777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'BBRC_Crystallization' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('BBRC_Crystallization',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('BBRC_Crystallization', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                   NVL (TO_CHAR (ANTISOLVENTRATE), ' ') "ANTISOLVENTRATE",
                   NVL (TO_CHAR (ADDITIONTEMP), ' ') "ADDITIONTEMP",
                   NVL (TO_CHAR (AGITATIONRATE), ' ') "AGITATIONRATE",
                   NVL (TO_CHAR (AGITATOR), ' ') "AGITATOR",
                   NVL (TO_CHAR (ANTISOLVENTSYS), ' ') "ANTISOLVENTSYS",
                   NVL (TO_CHAR (BATCHSIZE), ' ') "BATCHSIZE",
                   NVL (TO_CHAR (BATCHREACTSTREAM), ' ') "BATCHREACTSTREAM",
                   NVL (TO_CHAR (COOLINGDISTEMP), ' ') "COOLINGDISTEMP",
                   NVL (TO_CHAR (COOLINGISOTEMP), ' ') "COOLINGISOTEMP",
                   NVL (TO_CHAR (COOLINGRATE), ' ')    "COOLINGRATE",
                   NVL (TO_CHAR (CRYSTALTYPE), ' ') "CRYSTALTYPE",
                   NVL (TO_CHAR (DISTILLTIME), ' ') "DISTILLTIME",
                   NVL (TO_CHAR (DISTCONSTTEMP), ' ') "DISTCONSTTEMP",
                   NVL (TO_CHAR (DISTCONSTVOLUME), ' ') "DISTCONSTVOLUME",
                   NVL (TO_CHAR (DISTISOVOLUME), ' ')  "DISTISOVOLUME",
                   NVL (TO_CHAR (DV50), ' ')           "DV50",
                   NVL (TO_CHAR (FINALSOLVENT), ' ')   "FINALSOLVENT",
                   NVL (TO_CHAR (GEOMETRY), ' ')      "GEOMETRY",
                   NVL (TO_CHAR (CONSTRUCTIONMATERIAL), ' ') "CONSTRUCTIONMATERIAL",
                   NVL (TO_CHAR (REACTOROCCUPANCYISO), ' ') "REACTOROCCUPANCYISO",
                   NVL (TO_CHAR (REACTOROCCUPANCYT0), ' ') "REACTOROCCUPANCYT0",
                   NVL (TO_CHAR (RSSATURATION), ' ') "RSSATURATION",
                   NVL (TO_CHAR (SEED), ' ') "SEED",
                   NVL (TO_CHAR (SEEDBATCH), ' ') "SEEDBATCH",
                   NVL (TO_CHAR (SEEDTEMP), ' ') "SEEDTEMP",
                   NVL (TO_CHAR (SOLVENTSYSTEM), ' ') "SOLVENTSYSTEM",
                   NVL (TO_CHAR (DISTSTARTVOL), ' ') "DISTSTARTVOL",
                   NVL (TO_CHAR (SLURRYTEMP), ' ') "SLURRYTEMP"
                 FROM P_MPDBCRYSTALLIZATION_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  ("ANTISOLVENTRATE",
                                   "ADDITIONTEMP",
                                   "AGITATIONRATE",
                                   "AGITATOR",
                                   "ANTISOLVENTSYS",
                                   "BATCHSIZE",
                                   "BATCHREACTSTREAM",
                                   "COOLINGDISTEMP",
                                   "COOLINGISOTEMP",
                                   "COOLINGRATE",
                                   "CRYSTALTYPE",
                                   "DISTILLTIME",
                                   "DISTCONSTTEMP",
                                   "DISTCONSTVOLUME",
                                   "DISTISOVOLUME",
                                   "DV50",
                                   "FINALSOLVENT",
                                   "GEOMETRY",
                                   "CONSTRUCTIONMATERIAL",
                                   "REACTOROCCUPANCYISO",
                                   "REACTOROCCUPANCYT0",
                                   "RSSATURATION",
                                   "SEED",
                                   "SEEDBATCH",
                                   "SEEDTEMP",
                                   "SOLVENTSYSTEM",
                                   "DISTSTARTVOL",
                                   "SLURRYTEMP"
                                   ))
    
    ---- Experiment BBRC_Filtration -----------------------
            UNION ALL
            SELECT *
            FROM (SELECT 72177777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'BBRC_Filtration' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('BBRC_Filtration',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('BBRC_Filtration', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                   NVL (TO_CHAR (APIBATCHSIZE), ' ') "APIBATCHSIZE",
                   NVL (TO_CHAR (CAKEHEIGHT), ' ') "CAKEHEIGHT",
                   NVL (TO_CHAR (CAKERESISTANCE), ' ') "CAKERESISTANCE",
                   NVL (TO_CHAR (DELIQUORINGTIME), ' ') "DELIQUORINGTIME",
                   NVL (TO_CHAR (FILTERMATERIAL), ' ') "FILTERMATERIAL",
                   NVL (TO_CHAR (FILTRATIONAREA), ' ') "FILTRATIONAREA",
                   NVL (TO_CHAR (FILTRATIONFLUX), ' ') "FILTRATIONFLUX",
                   NVL (TO_CHAR (FILTRATIONTIME), ' ') "FILTRATIONTIME",
                   NVL (TO_CHAR (FILTRATIONVOL), ' ')    "FILTRATIONVOL",
                   NVL (TO_CHAR (FILTRATIONTYPE), ' ') "FILTRATIONTYPE"
    
    
                 FROM P_MPDBFILTRATION_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  ("APIBATCHSIZE",
                                   "CAKEHEIGHT",
                                   "CAKERESISTANCE",
                                   "DELIQUORINGTIME",
                                   "FILTERMATERIAL",
                                   "FILTRATIONAREA",
                                   "FILTRATIONFLUX",
                                   "FILTRATIONTIME",
                                   "FILTRATIONVOL",
                                   "FILTRATIONTYPE"
    
                                   ))
    
    ---- Experiment BBRC_Drying -----------------------
            UNION ALL
            SELECT *
            FROM (SELECT 72277777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'BBRC_Drying' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('BBRC_Drying',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('BBRC_Drying', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                   NVL (TO_CHAR (BATCHSIZE), ' ') "BATCHSIZE",
                   NVL (TO_CHAR (DRYERMATERIAL), ' ') "DRYERMATERIAL",
                   NVL (TO_CHAR (DRYERTYPE), ' ') "DRYERTYPE",
                   NVL (TO_CHAR (DRYERTEMP), ' ') "DRYERTEMP",
                   NVL (TO_CHAR (HUMIDITY), ' ') "HUMIDITY",
                   NVL (TO_CHAR (INITBEDHEIGHT), ' ') "INITBEDHEIGHT",
                   NVL (TO_CHAR (MAXDRYERJACKETTEMP), ' ') "MAXDRYERJACKETTEMP",
                   NVL (TO_CHAR (NSTEAMSWEEP), ' ') "NSTEAMSWEEP",
                   NVL (TO_CHAR (NSWEEPRATE), ' ')    "NSWEEPRATE",
                   NVL (TO_CHAR (RESHUFFLESCOUNT), ' ') "RESHUFFLESCOUNT",
                   NVL (TO_CHAR (TRAYSCOUNT), ' ') "TRAYSCOUNT",
                   NVL (TO_CHAR (UNLOADING), ' ') "UNLOADING",
                   NVL (TO_CHAR (VACUUM), ' ')  "VACUUM",
                   NVL (TO_CHAR (TRAYWEIGHT), ' ')  "TRAYWEIGHT"
                 FROM P_MPDBDRYING_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
                    UNPIVOT (calvalue
                               FOR calname
                               IN  ("BATCHSIZE",
                                   "DRYERMATERIAL",
                                   "DRYERTYPE",
                                   "DRYERTEMP",
                                   "HUMIDITY",
                                   "INITBEDHEIGHT",
                                   "MAXDRYERJACKETTEMP",
                                   "NSTEAMSWEEP",
                                   "NSWEEPRATE",
                                   "RESHUFFLESCOUNT",
                                   "TRAYSCOUNT",
                                   "UNLOADING",
                                   "VACUUM",
                                   "TRAYWEIGHT"
                                   ))
    
    ---- Experiment Jet Milling -----------------------
            UNION ALL
            SELECT *
            FROM (SELECT 72377777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'Jet Milling' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('Jet Milling',
                                                            'E')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('Jet Milling', 'E')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                   NVL (TO_CHAR (CRYSTCHANGE), ' ') "CRYSTCHANGE",
                   NVL (TO_CHAR (MILLDATE), ' ') "MILLDATE",
                   NVL (TO_CHAR (FEEDBATCH), ' ') "FEEDBATCH",
                   NVL (TO_CHAR (FEEDRATE), ' ') "FEEDRATE",
                   NVL (TO_CHAR (FEEDERTYPE), ' ') "FEEDERTYPE",
                   NVL (TO_CHAR (REGTYPE), ' ') "REGTYPE",
                   NVL (TO_CHAR (INPUTENERGY), ' ') "INPUTENERGY",
                   NVL (TO_CHAR (FEEDMASS), ' ') "FEEDMASS",
                   NVL (TO_CHAR (PRODUCTMASS), ' ')    "PRODUCTMASS",
                   NVL (TO_CHAR (MILLPRESSURE), ' ') "MILLPRESSURE",
                   NVL (TO_CHAR (MILLUSED), ' ') "MILLUSED",
                   NVL (TO_CHAR (MILLSITE), ' ') "MILLSITE",
                   NVL (TO_CHAR (PROJECTPHASE), ' ') "PROJECTPHASE",
                   NVL (TO_CHAR (PURPOSE), ' ') "PURPOSE",
                   NVL (TO_CHAR (RETAININMILL), ' ') "RETAININMILL",
                   NVL (TO_CHAR (VENTURIPRESSURE), ' ') "VENTURIPRESSURE",
                   NVL (TO_CHAR (YIELD), ' ') "YIELD"
                 FROM P_MPDBJETMILLING_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  ("CRYSTCHANGE",
                                   "MILLDATE",
                                   "FEEDBATCH",
                                   "FEEDRATE",
                                   "FEEDERTYPE",
                                   "REGTYPE",
                                   "INPUTENERGY",
                                   "FEEDMASS",
                                   "PRODUCTMASS",
                                   "MILLPRESSURE",
                                   "MILLUSED",
                                   "MILLSITE",
                                   "PROJECTPHASE",
                                   "PURPOSE",
                                   "RETAININMILL",
                                   "VENTURIPRESSURE",
                                   "YIELD"
                                   ))
    
    ---- Experiment PARTICLE_SIZE_LLS -----------------------
            UNION ALL
            SELECT *
            FROM (SELECT 72477777 + ROWNUM AS Sample_key,
                   sv.samplename AS sampleId,
                   sv.creator AS scientist,
                   sv.sampletype AS materialType,
                   sv.bmsbmtnumber AS bmsNum,
                   sv.excipientname AS EXCIPIENTNAME,
                   sv.batchnumber AS batchNum,
                   sv.sapnumber AS sapNum,
                   sv.otherid AS otherId,
                   sv.note AS note,
                   sv.creator AS createdBy,
                   sv.creationdate AS createdDate,
                   ' ' AS MODIFIEDBY,
                   'Y' AS STATUS,
                   sv.granulationtype AS granulation,
                   sv.blend AS blend,
                   sv.elnnumber AS elnNum,
                   ' ' AS CREATEDSITE,
                   ' ' AS STDBMS,
                   SAMPLE_DESC,
                   -- 'No' AS mergedSample,
                   --    experiment info
                   'PARTICLE_SIZE_LLS' AS EXPERIMENTNAME,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id ('PARTICLE_SIZE_LLS',
                                                            'P')
                      AS EXPERIMENTID,
                   PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE ('PARTICLE_SIZE_LLS', 'P')
                      AS experimentCode,
                   --  Results    Info
                   --p.calvalue,
                   -- p.calname,
                   0 AS REPNUM,
                   'ELN' AS dataSrc,
                   NVL (TO_CHAR (PERCTVOLUME), ' ') "Volume400768454",
                   NVL (TO_CHAR (RSDVOLUME), ' ') "RSD739802446",
                   NVL (TO_CHAR (RSDD4_3), ' ') "RSDD43628116377",
                   NVL (TO_CHAR (RSDD10), ' ') "RSDD10732557784",
                   NVL (TO_CHAR (RSDD50), ' ') "RSDD50154247269",
                   NVL (TO_CHAR (RSDD90), ' ') "RSDD90113493081",
                   NVL (TO_CHAR (RSDSPAN), ' ') "RSDSpan310776284",
                   NVL (TO_CHAR (RSDTRANSMISSION), ' ') "RSDTransmission545300776",
                   NVL (TO_CHAR (AVGVOLUME), ' ') "Average177631501",
                   NVL (TO_CHAR (AVGD4_3), ' ')    "AverageD43548349793",
                   NVL (TO_CHAR (AVGD10), ' ') "AverageD10419150647",
                   NVL (TO_CHAR (AVGD50), ' ') "AverageD50199586768",
                   NVL (TO_CHAR (AVGD90), ' ') "AverageD90527972031",
                   NVL (TO_CHAR (AVGSPAN), ' ') "AverageSpan571035535",
                   NVL (TO_CHAR (AVGTRANSMISSION), ' ')  "AverageTransmission83123406",
                   NVL (TO_CHAR (BELOW100MIC), ' ')           "Be0",
                   NVL (TO_CHAR (BELOW200MIC), ' ')   "820552619129056896",
                   NVL (TO_CHAR (BELOW30MIC), ' ')      "605439297559691008",
                   NVL (TO_CHAR (BELOW50MIC), ' ') "95020549693972224",
                   NVL (TO_CHAR (CALLPOINTTIME), ' ') "881510817934223744",
                   NVL (TO_CHAR (D3_2), ' ') "142275585808000816",
                   NVL (TO_CHAR (D4_3), ' ') "548899107614318720",
                   NVL (TO_CHAR (D10), ' ') "23417223190185356",
                   NVL (TO_CHAR (D50), ' ') "111854307403682400",
                   NVL (TO_CHAR (D90), ' ') "408804894250368832",
                   NVL (TO_CHAR (D95), ' ') "310354015043010752",
                   NVL (TO_CHAR (DATEOFANALYSIS), ' ') "903278078175493376",
                   NVL (TO_CHAR (FILENAME), ' ') "713192195458238080",
                   NVL (TO_CHAR (MAX), ' ') "514562416100610368",
                   NVL (TO_CHAR (MEANPARTICLESIZE), ' ') "914738326683865984",
                   NVL (TO_CHAR (MEANSQUAREWEIGHT), ' ') "745833788709097216",
                   NVL (TO_CHAR (MEDIAN), ' ') "204073862837270688",
                   NVL (TO_CHAR (METHODREF), ' ') "754776015495703552",
                   NVL (TO_CHAR (METHODVERSION), ' ') "472353106419791808",
                   NVL (TO_CHAR (MIN), ' ') "251447634879815136",
                   NVL (TO_CHAR (PARTICALSIZELLSRESULTS), ' ') "975699764423454464",
                   NVL (TO_CHAR (PARTICALSIZELLS), ' ') "815598026069164544",
                   NVL (TO_CHAR (SPAN), ' ') "820901034921496704",
                   NVL (TO_CHAR (TRANSMISSION), ' ') "129507261832775840"
                 FROM P_MPDBLaserDiffraction_OBJ  P,
                  P_MPDBSAMPLEINFO_OBJ S,
                   sample_view SV
             WHERE     P.OWNERID = S.OWNERID
                   AND P.CONTAINERID = S.CONTAINERID
                   AND P.PARENTID = S.PARENTID
                   AND P.VAULTID = S.VAULTID
                   AND S.CONTAINERID = SV.DOCUMENTID
                   AND S.SAMPLEID = SV.SampleName)
    
                    UNPIVOT (calvalue
                               FOR calname
                               IN  ("Volume400768454",
                                    "RSD739802446",
                                   "RSDD43628116377",
                                   "RSDD10732557784",
                                   "RSDD50154247269",
                                   "RSDD90113493081",
                                   "RSDSpan310776284",
                                   "RSDTransmission545300776",
                                   "Average177631501",
                                   "AverageD43548349793",
                                   "AverageD10419150647",
                                   "AverageD50199586768",
                                   "AverageD90527972031",
                                   "AverageSpan571035535",
                                   "AverageTransmission83123406",
                                   "Be0",
                                   "820552619129056896",
                                   "605439297559691008",
                                   "95020549693972224",
                                   "881510817934223744",
                                   "142275585808000816",
                                   "548899107614318720",
                                   "23417223190185356",
                                   "111854307403682400",
                                   "408804894250368832",
                                   "310354015043010752",
                                   "903278078175493376",
                                   "713192195458238080",
                                   "514562416100610368",
                                   "914738326683865984",
                                   "745833788709097216",
                                   "204073862837270688",
                                   "754776015495703552",
                                   "472353106419791808",
                                   "251447634879815136",
                                   "975699764423454464",
                                   "815598026069164544",
                                   "820901034921496704",
                                   "129507261832775840"
                                   ))
    
            ----------------------------------------------------------------------------------------------
            --Prod  only Experiments --------------------------------------------------------------------
    
                     UNION ALL
                     SELECT *
                       FROM (SELECT 71277777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    -- null as MODIFIEDDATE,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    'Erweka Flow | BPRD-DM0068'
                                       AS experimentTypeName,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Erweka Flow | BPRD-DM0068',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Erweka Flow | BPRD-DM0068',
                                       'E')
                                       AS experimentCode,
                                    --  3 AS experimentTypeId,
                                    --'ERWE' AS experimentCode,
                                    --  Results    Info
                                    -- p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    NVL (TO_CHAR (MASSFLOWRATE1), ' ')
                                       "calculatVolume1",
                                    NVL (TO_CHAR (MASSFLOWRATE2), ' ')  "calculatVolume2",
                                    NVL (TO_CHAR (MASSFLOWRATE3), ' ')     "calculatVolume3"
                               FROM P_MPDBSIEVEANALYSISBULKDEN_OBJ P,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    Sample_View SV,
                                    VAULTOBJECT_OBJ V
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName
                                    AND P.PARENTID = V.VAULTID
                                    AND V.TITLE = 'Erweka Flow') UNPIVOT (calvalue
                                                                 FOR calname
                                                                 IN  ("calculatVolume1",
                                                                     "calculatVolume2",
                                                                     "calculatVolume3"))
                     UNION ALL
                     SELECT *
                       FROM (SELECT 71377777 + ROWNUM AS Sample_key,
                                    sv.samplename AS sampleId,
                                    sv.creator AS scientist,
                                    sv.sampletype AS materialType,
                                    sv.bmsbmtnumber AS bmsNum,
                                    sv.excipientname AS EXCIPIENTNAME,
                                    sv.batchnumber AS batchNum,
                                    sv.sapnumber AS sapNum,
                                    sv.otherid AS otherId,
                                    sv.note AS note,
                                    sv.creator AS createdBy,
                                    sv.creationdate AS createdDate,
                                    ' ' AS MODIFIEDBY,
                                    -- null as MODIFIEDDATE,
                                    'Y' AS STATUS,
                                    sv.granulationtype AS granulation,
                                    sv.blend AS blend,
                                    sv.elnnumber AS elnNum,
                                    ' ' AS CREATEDSITE,
                                    ' ' AS STDBMS,
                                    SAMPLE_DESC,
                                    --'No' AS mergedSample,
                                    --    experiment info
                                    'Scott''s Volumeter BULK Density | BPRD-DM-0073'
                                       AS EXPERIMENTNAME,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_type_id (
                                       'Scott''s Volumeter BULK Density | BPRD-DM-0073',
                                       'E')
                                       AS EXPERIMENTID,
                                    PDMP_SUPPORT_PKG.get_EXPERIMENT_CODE (
                                       'Scott''s Volumeter BULK Density | BPRD-DM-0073',
                                       'E')
                                       AS experimentCode,
                                    --  Results    Info
                                    --  p.calvalue,
                                    -- p.calname,
                                    0 AS REPNUM,
                                    'ELN' AS dataSrc,
                                    CASE
                                       WHEN SUBSTR (CALBULKDENSITYREP1, 1, 1) =
                                               '.'
                                       THEN
                                          '0' || TO_CHAR (CALBULKDENSITYREP1)
                                       ELSE
                                          NVL (TO_CHAR (CALBULKDENSITYREP1), ' ')
                                    END
                                       "bulkDensity1",
                                    CASE
                                       WHEN SUBSTR (CALBULKDENSITYREP2, 1, 1) =
                                               '.'
                                       THEN
                                          '0' || TO_CHAR (CALBULKDENSITYREP2)
                                       ELSE
                                          NVL (TO_CHAR (CALBULKDENSITYREP2), ' ')
                                    END
                                       "bulkDensity2"
                               FROM P_MPDBSIEVEANALYSISBULKDEN_OBJ p,
                                    P_MPDBSAMPLEINFO_OBJ S,
                                    sample_view SV,
                                    VAULTOBJECT_OBJ V
                              WHERE     P.OWNERID = S.OWNERID
                                    AND P.CONTAINERID = S.CONTAINERID
                                    AND P.PARENTID = S.PARENTID
                                    AND P.VAULTID = S.VAULTID
                                    AND S.CONTAINERID = SV.DOCUMENTID
                                    AND S.SAMPLEID = SV.SampleName
                                    AND P.PARENTID = V.VAULTID
                                    AND V.TITLE =
                                           'Scott''s Volumeter Bulk Density') UNPIVOT (calvalue
                                                                              FOR calname
                                                                              IN  ("bulkDensity1",
                                                                                  "bulkDensity2"))
    
                     UNION ALL
                     SELECT *
                       FROM (WITH Sieve
                                  AS (SELECT   71477777
                                             + RANK ()
                                                  OVER (ORDER BY
                                                           p.samplename,
                                                           p.bmsbmtnumber,
                                                           p.batchnumber,
                                                           p.sampletype,
                                                           p.geometricmean,
                                                           p.geometricstdev,
                                                           p.arithmeticmean,
                                                           p.arithmeticstdev)
                                                AS sample_key,
                                             sv.samplename AS sampleid,
                                             -- sampleId+bmsNum+batchNum+materialType+geoMean+geoSD+arithMean+to_number(arithSD )as sample_key,
                                             sv.creator AS scientist,
                                             sv.sampletype AS materialtype,
                                             sv.bmsbmtnumber AS bmsnum,
                                             sv.excipientname AS excipientname,
                                             sv.batchnumber AS batchnum,
                                             sv.sapnumber AS sapnum,
                                             sv.otherid AS otherid,
                                             sv.note AS note,
                                             sv.creator AS createdby,
                                             sv.creationdate AS createddate,
                                             ' ' AS modifiedby,
                                             --  null as modifieddate,
                                             'Y' AS status,
                                             sv.granulationtype AS granulation,
                                             sv.blend AS blend,
                                             sv.elnnumber AS elnnum,
                                             ' ' AS createdsite,
                                             ' ' AS stdbms,
                                             NULL sample_desc,
                                             ROW_NUMBER ()
                                             OVER (PARTITION BY p.samplename,
                                                                p.bmsbmtnumber,
                                                                p.batchnumber,
                                                                p.sampletype,
                                                                p.geometricmean,
                                                                p.geometricstdev,
                                                                p.arithmeticmean,
                                                                p.arithmeticstdev
                                                   ORDER BY geometricmean)
                                                geomean_rank,
                                             ROW_NUMBER ()
                                             OVER (PARTITION BY p.samplename,
                                                                p.bmsbmtnumber,
                                                                p.batchnumber,
                                                                p.sampletype,
                                                                p.geometricmean,
                                                                p.geometricstdev,
                                                                p.arithmeticmean,
                                                                p.arithmeticstdev
                                                   ORDER BY geometricstdev)
                                                geosd_rank,
                                             ROW_NUMBER ()
                                             OVER (PARTITION BY p.samplename,
                                                                p.bmsbmtnumber,
                                                                p.batchnumber,
                                                                p.sampletype,
                                                                p.geometricmean,
                                                                p.geometricstdev,
                                                                p.arithmeticmean,
                                                                p.arithmeticstdev
                                                   ORDER BY arithmeticmean)
                                                arithmean_rank,
                                             ROW_NUMBER ()
                                             OVER (PARTITION BY p.samplename,
                                                                p.bmsbmtnumber,
                                                                p.batchnumber,
                                                                p.sampletype,
                                                                p.geometricmean,
                                                                p.geometricstdev,
                                                                p.arithmeticmean,
                                                                p.arithmeticstdev
                                                   ORDER BY arithmeticstdev)
                                                arithsd_rank,
                                             p.geometricmean,
                                             p.geometricstdev,
                                             p.arithmeticmean,
                                             p.arithmeticstdev,
                                             'Sieve Analysis Data | BPRD-DM-0065'
                                                AS experimenttypename,
                                             pdmp_support_pkg.get_experiment_type_id (
                                                'Sieve Analysis Data | BPRD-DM-0065',
                                                'E')
                                                AS experimentid,
                                             pdmp_support_pkg.get_experiment_code (
                                                'Sieve Analysis Data | BPRD-DM-0065',
                                                'E')
                                                AS experimentcode,
                                             -- P2.MESHNUMBER,
                                             -- P2.RETAINEDPERCENT,
                                             0 AS repnum,
                                             'ELN' AS datasrc,
                                             TO_CHAR (
                                                   'micron'
                                                || RANK ()
                                                   OVER (
                                                      PARTITION BY p.samplename,
                                                                   p.bmsbmtnumber,
                                                                   p.batchnumber,
                                                                   p.sampletype,
                                                                   p.geometricmean,
                                                                   p.geometricstdev,
                                                                   p.arithmeticmean,
                                                                   p.arithmeticstdev
                                                      ORDER BY p2.ID))
                                                mcrn_calname,
                                             CASE
                                                WHEN TO_CHAR (p2.diameter)
                                                        IS NULL
                                                THEN
                                                   '  '
                                                WHEN p2.retainedpercent IS NULL
                                                THEN
                                                   '  '
                                                ELSE
                                                   TRIM (
                                                         TO_CHAR (p2.diameter,
                                                                  '99999999.9')
                                                      || ':'
                                                      || p2.retainedpercent)
                                             END
                                                mcrn_calvalue
                                        FROM p_mpdbsieveanalysisbulkden_obj p,
                                             p_mpdbsieveanalysis_obj p2,
                                             p_mpdbsampleinfo_obj s,
                                             sample_view sv,
                                             vaultobject_obj v
                                       WHERE     p.ownerid = s.ownerid
                                             AND p.containerid = s.containerid
                                             AND p.parentid = s.parentid
                                             AND p.vaultid = s.vaultid
                                             AND p2.ownerid = p.ownerid
                                             AND p2.containerid = p.containerid
                                             AND p2.parentid = p.parentid
                                             AND s.containerid = sv.documentid
                                             AND s.sampleid = sv.samplename
                                             AND p.parentid = v.vaultid
                                             AND v.title = 'Sieve Analysis' -- AND bmsbmtnumber = 'BMS-562247'
                                              -- AND batchnumber = '96752-017 - 9'
                                     )
                             --union all
                             SELECT sample_key,
                                    sampleid,
                                    scientist,
                                    materialtype,
                                    bmsnum,
                                    excipientname,
                                    batchnum,
                                    sapnum,
                                    otherid,
                                    note,
                                    createdby,
                                    createddate,
                                    modifiedby,
                                    -- modifieddate,
                                    status,
                                    granulation,
                                    blend,
                                    elnnum,
                                    createdsite,
                                    stdbms,
                                    sample_desc,
                                    experimenttypename AS EXPERIMENTNAME,
                                    experimentid,
                                    experimentcode,
                                    repnum,
                                    datasrc,
                                    calname,
                                    calvalue
                               FROM (SELECT sample_key,
                                            sampleid,
                                            scientist,
                                            materialtype,
                                            bmsnum,
                                            excipientname,
                                            batchnum,
                                            sapnum,
                                            otherid,
                                            note,
                                            createdby,
                                            createddate,
                                            modifiedby,
                                            -- modifieddate,
                                            status,
                                            granulation,
                                            blend,
                                            elnnum,
                                            createdsite,
                                            stdbms,
                                            sample_desc,
                                            experimenttypename,
                                            experimentid,
                                            experimentcode,
                                            geomean_rank,
                                            arithsd_rank,
                                            geosd_rank,
                                            arithmean_rank,
                                            --calname,
                                            -- calvalue,
                                            repnum,
                                            datasrc,
                                            NVL (TO_CHAR (geometricmean), '  ')
                                               "geoMean", -- this is special case -- need to keep 2 spaces
                                            NVL (TO_CHAR (geometricstdev), '  ')
                                               "geoSD", -- this is special case -- need to keep 2 spaces
                                            NVL (TO_CHAR (arithmeticmean), '  ')
                                               "arithMean", -- this is special case -- need to keep 2 spaces
                                            NVL (TO_CHAR (arithmeticstdev), '  ')
                                               "arithSD" -- this is special case -- need to keep 2 spaces
                                       FROM Sieve) UNPIVOT (calvalue
                                                   FOR calname
                                                   IN  ("geoMean",
                                                       "geoSD",
                                                       "arithMean",
                                                       "arithSD"))
                              --WHERE     bmsnum = 'BMS-562247'
                              -- AND batchnum = '96752-017 - 9'
                              WHERE (    geomean_rank = 1
                                     AND geosd_rank = 1
                                     AND arithmean_rank = 1
                                     AND arithsd_rank = 1)
                             UNION ALL
                             SELECT sample_key,
                                    sampleid,
                                    scientist,
                                    materialtype,
                                    bmsnum,
                                    excipientname,
                                    batchnum,
                                    sapnum,
                                    otherid,
                                    note,
                                    createdby,
                                    createddate,
                                    modifiedby,
                                    --  modifieddate,
                                    status,
                                    granulation,
                                    blend,
                                    elnnum,
                                    createdsite,
                                    stdbms,
                                    sample_desc,
                                    experimenttypename,
                                    experimentid,
                                    experimentcode,
                                    repnum,
                                    'ELN' datasrc,
                                    mcrn_calname AS calname,
                                    mcrn_calvalue AS calvalue
                               FROM Sieve)
    
           --- ONLY PROD EXPERIEMNT
    
               )
    */
    COMMIT;
    /* if successful, copy data to ELN result MV table */
    EXECUTE 'TRUNCATE TABLE orcl.eln_sample_result_mv';
    INSERT INTO orcl.eln_sample_result_mv
    SELECT
        *
        FROM orcl.eln_sample_result_temp;
    COMMIT;
    v_message := CONCAT_WS('', v_message, aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_crlf'), 'end: ', aws_oracle_ext.TO_CHAR(aws_oracle_ext.SYSDATE(), 'MM/DD/YY HH24:MI:SS'));
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    v_message :=
                v_message
             || gc_crlf
             || 'execution time: '
             || (DBMS_UTILITY.get_time - v_begin_time) / 100
             || ' secs.'
    */
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_message);
    EXCEPTION
        WHEN others THEN
            ROLLBACK;
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack :=
                           'ERROR Stack: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_stack
                        || gc_crlf
                        || 'ERROR Backtrace: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_backtrace
            */
            /* dbms_output.put_line('refresh_eln_experiments fail reached'); */
            CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'F'::TEXT, v_error_stack);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.mviews_refresh_pkg$refresh_mpdb_experiments()
AS 
$BODY$
/* ----------------------------------------------------------------------------- */
/* Name:refresh_mpdb_experiments */
/* Description: Procedure to read data from  MPDB tables and refresh */
/* pdmp_sample_result_mv. */
/* Output: None. */
/* Created: 3/11/2015, krishna L */
/* ----------------------------------------------------------------------------- */
DECLARE
    v_begin_time DOUBLE PRECISION
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    := DBMS_UTILITY.get_time
    */;
    v_date TIMESTAMP(0) WITHOUT TIME ZONE := aws_oracle_ext.SYSDATE();
    v_message CHARACTER VARYING(3000);
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'refresh_mpdb_experiments';
BEGIN
    PERFORM ORCL.MVIEWS_REFRESH_PKG$Init();
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'S'::TEXT, NULL);
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_SNAPSHOT.REFRESH(VARCHAR2,VARCHAR2,VARCHAR2,BOOLEAN,BOOLEAN,BINARY_INTEGER,BINARY_INTEGER,BINARY_INTEGER,BOOLEAN,BOOLEAN,BOOLEAN) function. Use suitable function or create user defined function.]
    DBMS_MVIEW.REFRESH ('pdmp_sample_result_mv', 'C')
    */
    v_message := CONCAT_WS('', v_message, aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_crlf'), 'end: ', aws_oracle_ext.TO_CHAR(aws_oracle_ext.SYSDATE(), 'MM/DD/YY HH24:MI:SS'));
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    v_message :=
                v_message
             || gc_crlf
             || 'execution time: '
             || (DBMS_UTILITY.get_time - v_begin_time) / 100
             || ' secs.'
    */
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_message);
    EXCEPTION
        WHEN others THEN
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack :=
                           'ERROR Stack: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_stack
                        || gc_crlf
                        || 'ERROR Backtrace: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_backtrace
            */
            CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'F'::TEXT, v_error_stack);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.mviews_refresh_pkg$refresh_pdlim_experiments()
AS 
$BODY$
/* ----------------------------------------------------------------------------- */
/* Name: refresh_pdlim_experiments */
/* Description: Procedure to read  data from PDLIM  tables and */
/* load pdlims_sample_result_mv_tbl . */
/* Output: None. */
DECLARE
    v_begin_time DOUBLE PRECISION
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    := DBMS_UTILITY.get_time
    */;
    v_date TIMESTAMP(0) WITHOUT TIME ZONE := aws_oracle_ext.SYSDATE();
    v_message CHARACTER VARYING(3000);
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'refresh_pdlim_experiments';
    /* v_count number; */
    v_Experiments CHARACTER VARYING(200);
    v_sql CHARACTER VARYING(4000);
BEGIN
    PERFORM ORCL.MVIEWS_REFRESH_PKG$Init();
    SELECT
        config_value
        INTO STRICT v_Experiments
        FROM orcl.mv_config
        WHERE config_name = 'Pdlim_experiment';
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'S'::TEXT, NULL);
    INSERT INTO orcl.bkp_pdlim
    SELECT
        *
        FROM orcl.pdlims_sample_result_mv_tbl;
    COMMIT;
    EXECUTE 'TRUNCATE TABLE orcl.pdlims_sample_result_mv_tbl';
    v_sql := CONCAT_WS('', '
insert into pdlims_sample_result_mv_tbl
SELECT DISTINCT a.sample_number + 1111111 sample_key,
                TO_CHAR (a.sample_number) AS sampleid,
                '' '' sample_desc,
                '' '' AS scientist,
                d.x_product_type AS materialtype,
                a.product AS bmsnum,
                '' '' AS excipientname,
                a.x_batch_num AS batchnum,
                '' '' AS sapnum,
                a.x_project_code AS otherid,
                a.x_comments AS note,
                c.entered_on AS createddate, ---These date coming from results but  MPDB this date coming from Sample Info
                c.entered_by AS createdby,
                '' '' AS modifiedby,
                SYSDATE AS modifieddate,
                ''Y'' AS status,
                '' '' AS granulation,
                '' '' AS blend,
                '' '' AS elnnum,
                '' '' AS createdsite,
                '' '' AS stdbms,
                b.analysis AS experimentname,
                c.name, ---> this is for  etl Verification if any values not matchig configuration table or incorrect display value
                c.formatted_entry AS calvalue,
                0 AS repnum,
                ''pdlims'' AS datasrc
  FROM ((sample@to_pdlim a
         LEFT JOIN test@to_pdlim b ON a.sample_number = b.sample_number)
        LEFT JOIN result@to_pdlim c
           ON     b.sample_number = c.sample_number
              AND b.test_number = c.test_number)
       LEFT JOIN product@to_pdlim d ON a.product = d.name
 WHERE b.analysis IN (', v_Experiments, ')');
    /*
    [5334 - Severity CRITICAL - Unable to convert statements with dynamic SQL statement. Please perform a manual conversion.]
    EXECUTE IMMEDIATE v_sql
    */
    COMMIT;
    EXECUTE 'TRUNCATE TABLE orcl.bkp_pdlim';
    /* DBMS_MVIEW.REFRESH('pdlims_sample_result_mv','C'); */
    v_message := CONCAT_WS('', v_message, aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_crlf'), 'end: ', aws_oracle_ext.TO_CHAR(aws_oracle_ext.SYSDATE(), 'MM/DD/YY HH24:MI:SS'));
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    v_message :=
                v_message
             || gc_crlf
             || 'execution time: '
             || (DBMS_UTILITY.get_time - v_begin_time) / 100
             || ' secs.'
    */
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_message);
    EXCEPTION
        WHEN others THEN
            ROLLBACK;
            /* select count(1) */
            /* into v_count */
            /* from pdlims_sample_result_mv_tbl; */
            /* */
            /* if v_count > 0 then */
            /* delete from pdlims_sample_result_mv_tbl */
            /* commit; */
            /* end if; */
            /* get the old copy  to retreve previously available data */
            INSERT INTO orcl.pdlims_sample_result_mv_tbl
            SELECT
                *
                FROM orcl.bkp_pdlim;
            COMMIT;
            /* This should be emapty after run completes */
            EXECUTE 'TRUNCATE TABLE orcl.bkp_pdlim';
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack :=
                           'ERROR Stack: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_stack
                        || gc_crlf
                        || 'ERROR Backtrace: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_backtrace
            */
            CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'F'::TEXT, v_error_stack);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.mviews_refresh_pkg$refresh_sample_result_mv()
AS 
$BODY$
/* ----------------------------------------------------------------------------- */
/* Name:refresh_eln_experiments */
/* Description: Procedure to load data from ELN tables into */
/* ELN_Experiments table. */
/* Output: None. */
/* ------------------------------------------------------------- */
DECLARE
    v_begin_time DOUBLE PRECISION
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    := DBMS_UTILITY.get_time
    */;
    v_date TIMESTAMP(0) WITHOUT TIME ZONE := aws_oracle_ext.SYSDATE();
    v_message CHARACTER VARYING(3000);
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'refresh_sample_result_mv';
BEGIN
    PERFORM ORCL.MVIEWS_REFRESH_PKG$Init();
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'S'::TEXT, NULL);
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_SNAPSHOT.REFRESH(VARCHAR2,VARCHAR2,VARCHAR2,BOOLEAN,BOOLEAN,BINARY_INTEGER,BINARY_INTEGER,BINARY_INTEGER,BOOLEAN,BOOLEAN,BOOLEAN) function. Use suitable function or create user defined function.]
    DBMS_MVIEW.REFRESH ('sample_result_mv', 'C')
    */
    v_message := CONCAT_WS('', v_message, aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_crlf'), 'end: ', aws_oracle_ext.TO_CHAR(aws_oracle_ext.SYSDATE(), 'MM/DD/YY HH24:MI:SS'));
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    v_message :=
                v_message
             || gc_crlf
             || 'execution time: '
             || (DBMS_UTILITY.get_time - v_begin_time) / 100
             || ' secs.'
    */
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_message);
    /* dbms_output.put_line('refresh_sample_result_mv complete reached'); */
    EXCEPTION
        WHEN others THEN
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack :=
                           'ERROR Stack: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_stack
                        || gc_crlf
                        || 'ERROR Backtrace: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_backtrace
            */
            RAISE DEBUG USING MESSAGE = 'refresh_sample_result_mv fail reached';
            CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'F'::TEXT, v_error_stack);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.mviews_refresh_pkg$run_all_loads()
AS 
$BODY$
/* ----------------------------------------------------------------------------- */
/* Name: run_all_sod */
/* Description: Procedure which runs all Mview  procedures. */
/* Input: */
/* */
/* Output: None. */
/* ----------------------------------------------------------------------------- */
DECLARE
    v_begin_time DOUBLE PRECISION
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    := DBMS_UTILITY.get_time
    */;
    /* v_date date := sysdate; */
    v_message CHARACTER VARYING(3000);
    /* -v_message varchar2(3); */
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'run_all_loads';
/* V_start_time varchar2(3000) := to_char(sysdate, 'MM/DD/YY HH24:MI:SS'); */
BEGIN
    PERFORM ORCL.MVIEWS_REFRESH_PKG$Init();
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'S'::TEXT, NULL);
    CALL orcl.mviews_refresh_pkg$refresh_mpdb_experiments();
    CALL orcl.mviews_refresh_pkg$refresh_pdlim_experiments();
    CALL orcl.mviews_refresh_pkg$refresh_eln_experiments();
    CALL orcl.mviews_refresh_pkg$refresh_sample_result_mv();
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_STATS.GATHER_TABLE_STATS(VARCHAR2,VARCHAR2,VARCHAR2,NUMBER,BOOLEAN,VARCHAR2,NUMBER,VARCHAR2,BOOLEAN,VARCHAR2,VARCHAR2,VARCHAR2,BOOLEAN,VARCHAR2,BOOLEAN,DBMS_STATS.CCONTEXT,VARCHAR2) function. Use suitable function or create user defined function.]
    DBMS_STATS.gather_table_stats (USER, 'sample_result_mv')
    */
    /* truncate table ELN_sample_result_mv12; */
    v_message := CONCAT_WS('', v_message, aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_crlf'), 'end: ', aws_oracle_ext.TO_CHAR(aws_oracle_ext.SYSDATE(), 'MM/DD/YY HH24:MI:SS'));
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.GET_TIME() function. Use suitable function or create user defined function.]
    v_message :=
                v_message
             || gc_crlf
             || 'execution time: '
             || (DBMS_UTILITY.get_time - v_begin_time) / 100
             || ' secs.'
    */
    CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_message);
    RAISE DEBUG USING MESSAGE = 'Status: run_all_Loads end reached';
    /* pdmp_support_pkg.send_email('krishna.lakamsani@bms.com', 'Null', v_messag'); */
    EXCEPTION
        WHEN others THEN
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack :=
                           'ERROR Stack: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_stack
                        || gc_crlf
                        || 'ERROR Backtrace: '
                        || gc_crlf
                        || DBMS_UTILITY.format_error_backtrace
            */
            CALL orcl.pdmp_support_pkg$log_event(aws_oracle_ext.get_package_variable('ORCL', 'MVIEWS_REFRESH_PKG', 'gc_pkg_name'), c_proc_name, 'C'::TEXT, v_error_stack);
/* pdmp_support_pkg.send_email('krishna.lakamsani@bms.com', null, 'v_error_stack'); */
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.my_pkg$get_person(
     IN p_person_id DOUBLE PRECISION)
AS 
$BODY$
DECLARE
    v_error_code DOUBLE PRECISION;
    v_full_name ORCL.DIM_PERSON.full_name%TYPE := NULL;
BEGIN
    SELECT
        full_name
        INTO STRICT v_full_name
        FROM orcl.dim_person
        WHERE person_id = p_person_id;

    IF v_full_name IS NULL OR v_full_name = '' THEN
        RAISE DEBUG USING MESSAGE = CONCAT_WS('', 'No such Person - ', p_person_id);
    ELSE
        RAISE DEBUG USING MESSAGE = CONCAT_WS('', 'Person Name - ', v_full_name);
    END IF;
    EXCEPTION
        WHEN others THEN
            /*
            [5602 - Severity CRITICAL - PostgreSQL error code type is not a number and is incompatible with the number type variables
                    . Please, migrate the assignment of SQLCODE manually.]
            v_error_code := SQLCODE
            */
            RAISE DEBUG USING MESSAGE = CONCAT_WS('', 'Error - ', v_error_code);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.myprint_proc(
     IN aa orcl.numarray)
AS 
$BODY$
DECLARE
    i CHARACTER VARYING(15);
BEGIN
    i := array_lower(aa, 1)::TEXT;

    WHILE i IS NOT NULL LOOP
        RAISE DEBUG USING MESSAGE = CONCAT_WS('', aa[i], '  ', i);
        i := aws_oracle_ext.next(aa, i)::TEXT;
    END LOOP;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.myproc(
     IN val1 DOUBLE PRECISION DEFAULT 1, 
     IN val2 DOUBLE PRECISION DEFAULT 2, 
     INOUT val3 DOUBLE PRECISION DEFAULT NULL)
AS 
$BODY$
BEGIN
    val3 := val1 + val2;
    RAISE DEBUG USING MESSAGE = val3::TEXT;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.pdmp_support_pkg$log_event(
     IN pi_pkg_name TEXT, 
     IN pi_proc_name TEXT, 
     IN pi_status TEXT, 
     IN pi_log_message TEXT)
AS 
$BODY$
DECLARE
    /*
    [5571 - Severity CRITICAL - Unable to convert AutonomousTransaction pragma. Check converted code for perfomance issues and rewrite it manually, if necessary.]
    pragma autonomous_transaction
    */
    /* ----------------------------------------------------------------------------- */
    /* Name: log_event */
    /* Description: Logs an event performed by this package into MV_audit_error_log */
    /* table. */
    /* */
    /* Input:  pi_pkg_name: Req, Package message came from. */
    /* pi_proc_name: Req, Procedure or function message came from. */
    /* pi_log_message: Req, Message to be logged. */
    /* Output: None. */
    /* */
    /* Created: 03/12/2015, Krishna L. */
    /* Modified: */
    /* ----------------------------------------------------------------------------- */
    v_default_email CHARACTER VARYING(256);
    v_error_stack CHARACTER VARYING(5000);
    c_proc_name CHARACTER VARYING(30) := 'LOG_EVENT';
    c_ind BOOLEAN :=
    CASE UPPER(pi_status)
        WHEN 'S' THEN TRUE
        ELSE FALSE
    END;
BEGIN
    PERFORM ORCL.PDMP_SUPPORT_PKG$Init();

    IF c_ind THEN
        INSERT INTO orcl.mv_audit_error_log
        VALUES (pi_pkg_name, pi_proc_name, aws_oracle_ext.SYSDATE(), NULL, pi_status, NULL, NULL);
        /* dbms_output.put_line('Status:' ||pi_pkg_name||--|| pi_proc_name||---|| pi_status); */
        COMMIT;
    ELSE
        UPDATE orcl.mv_audit_error_log
        SET end_time = aws_oracle_ext.SYSDATE(), status = pi_status, audit_error_msg = pi_log_message
            WHERE LOWER(procedure_name) = pi_proc_name AND LOWER(package_name) = 'mviews_refresh_pkg' AND start_time = (SELECT
                MAX(start_time)
                FROM orcl.mv_audit_error_log
                WHERE LOWER(procedure_name) = pi_proc_name);
        /* dbms_output.put_line('Status:' || pi_status); */
        COMMIT;
    /* dbms_output.put_line('Status:' ||pi_pkg_name||'--'|| pi_proc_name||'---'|| pi_status); */
    END IF;
    /* MV_MV_audit_error_log */
    EXCEPTION
        WHEN others THEN
            /*
            [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_STACK() function. Use suitable function or create user defined function., 5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_UTILITY.FORMAT_ERROR_BACKTRACE() function. Use suitable function or create user defined function.]
            v_error_stack := 'ERROR Stack: ' || gc_crlf || dbms_utility.format_error_stack || gc_crlf ||
                                'ERROR Backtrace: ' || gc_crlf || dbms_utility.format_error_backtrace
            */
            RAISE DEBUG USING MESSAGE = v_error_stack::TEXT;
            /* pdmp_support_pkg.send_email(v_default_email, gc_pkg_name||'.'||c_proc_name || ' ERROR', v_error_stack); */
            RAISE;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.seed_data_broker()
AS 
$BODY$
BEGIN
    FOR i IN 1..10000 LOOP
        INSERT INTO orcl.broker (brokerid, brokername, ticker_lookup, entity, shortname, entitytypeid)
        VALUES (i, CONCAT_WS('', 'broker_', i), CONCAT_WS('', 'ticker_lookup_', i), CONCAT_WS('', 'entity_', i), CONCAT_WS('', 'nm_', i), CAST (aws_oracle_ext.dbms_random$value(2, 50) AS NUMERIC(38)));
    END LOOP;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.seed_data_entity_type()
AS 
$BODY$
BEGIN
    FOR i IN 1..50 LOOP
        INSERT INTO orcl.entity_type (entitytypeid, orcl.entity_type)
        VALUES (i, CONCAT_WS('', 'entity_type_', i));
    END LOOP;
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.send_mail(
     IN p_to TEXT, 
     IN p_from TEXT, 
     IN p_message TEXT, 
     IN p_smtp_host TEXT, 
     IN p_smtp_port DOUBLE PRECISION DEFAULT 25)
AS 
$BODY$
DECLARE
    l_mail_conn aws_oracle_ext.utl_smtp$connection;
BEGIN
    l_mail_conn := aws_oracle_ext.utl_smtp$open_connection(p_smtp_host, p_smtp_port);
    PERFORM aws_oracle_ext.utl_smtp$helo(l_mail_conn, p_smtp_host);
    PERFORM aws_oracle_ext.utl_smtp$starttls(l_mail_conn);
    PERFORM aws_oracle_ext.utl_smtp$mail(l_mail_conn, p_from);
    PERFORM aws_oracle_ext.utl_smtp$rcpt(l_mail_conn, p_to);
    PERFORM aws_oracle_ext.utl_smtp$data(l_mail_conn, CONCAT_WS('', p_message, CRLF, CRLF));
    PERFORM aws_oracle_ext.utl_smtp$quit(l_mail_conn);
END;
$BODY$
LANGUAGE plpgsql;



CREATE PROCEDURE orcl.update_dim_mandate_conflicts()
AS 
$BODY$
DECLARE
    cur CURSOR FOR
    SELECT DISTINCT
        mandate_dim_id, participant_id
        FROM stg_possible_conflicts
        ORDER BY mandate_dim_id;
    up_down_cur CURSOR FOR
    SELECT
        a.mandate_dim_id, a.mandate_id, a.participant_id, a.pro_id, a.status, a.date_to, LENGTH(b.target_territories_included::TEXT) AS country_length
        FROM dim_mandate AS a
        JOIN dim_mandate_detail AS b
            ON a.mandate_dim_id = b.mandate_dim_id
        WHERE a.status IN ('A', 'H') AND a.date_to > aws_oracle_ext.SYSDATE() AND
        /* The following only looks at the new rows inserted into dim_mandate_detail, acts as a delta */
        b.mandate_coverage_status IS NULL
        ORDER BY a.mandate_dim_id;
    v_conflict_mandate_dim_id CHARACTER VARYING(4000) := '1000';
    v_country_length DOUBLE PRECISION := 0;
    v_max_date TIMESTAMP(0) WITHOUT TIME ZONE;
BEGIN
    /* update dim_mandate_detail */
    /* set conflict_mandate_dim_id = null; */
    /* commit; */
    EXECUTE 'TRUNCATE TABLE stg_possible_conflicts';
    INSERT INTO stg_possible_conflicts
    SELECT
        a.mandate_dim_id, a.participant_id, COALESCE(a.pro_id, b.pro_id) AS pro_id
        FROM dim_mandate AS a
        JOIN dim_mandate AS b
            ON a.participant_id = b.participant_id AND a.pro_id != b.pro_id
        WHERE a.valid_to > aws_oracle_ext.SYSDATE() AND b.valid_to > aws_oracle_ext.SYSDATE() AND a.status IN ('A', 'H') AND b.status IN ('A', 'H') AND a.date_to > aws_oracle_ext.SYSDATE() AND b.date_to > aws_oracle_ext.SYSDATE() AND
        /* The following only looks at the new rows inserted into dim_mandate_detail, acts as a delta */
        EXISTS (SELECT
            1
            FROM dim_mandate AS x
            JOIN dim_mandate_detail AS y
                ON x.mandate_dim_id = y.mandate_dim_id
            WHERE x.participant_id = a.participant_id AND y.conflict_mandate_dim_id IS NULL);
    COMMIT;
    EXECUTE 'TRUNCATE TABLE stg_conflict_country_codes';
    INSERT INTO stg_conflict_country_codes
    SELECT
        a.mandate_dim_id, a.target_territories, b.participant_id, b.pro_id, TRIM(aws_oracle_ext.regexp_substr((target_territories_included)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels.column_value::NUMERIC)::INTEGER)::NUMERIC)) AS target_territories_included, aws_oracle_ext.SYSDATE() AS dw_created_date
        FROM dim_mandate_detail AS a, stg_possible_conflicts AS b,
        /* Transformer error occurred */
        WHERE a.mandate_dim_id = b.mandate_dim_id;
    COMMIT;
    /*
    [5340 - Severity CRITICAL - PostgreSQL doesn't support the SYS.DBMS_STATS.GATHER_TABLE_STATS(VARCHAR2,VARCHAR2,VARCHAR2,NUMBER,BOOLEAN,VARCHAR2,NUMBER,VARCHAR2,BOOLEAN,VARCHAR2,VARCHAR2,VARCHAR2,BOOLEAN,VARCHAR2,BOOLEAN,DBMS_STATS.CCONTEXT,VARCHAR2) function. Use suitable function or create user defined function.]
    dbms_stats.gather_table_stats('SX_EDW_DIM','STG_CONFLICT_COUNTRY_CODES',cascade=>TRUE,no_invalidate=>FALSE)
    */
    FOR rec IN cur LOOP
        v_conflict_mandate_dim_id := '1000';
        EXECUTE 'TRUNCATE TABLE stg_conflicts';
        EXECUTE 'TRUNCATE TABLE stg_last_conflict_check';
        EXECUTE 'TRUNCATE TABLE stg_final_mandate_conflicts';
        INSERT INTO stg_conflicts
        SELECT DISTINCT
            a.*
            FROM (SELECT
                *
                FROM stg_conflict_country_codes
                WHERE participant_id = rec.participant_id) AS a
            JOIN (SELECT
                *
                FROM stg_conflict_country_codes
                WHERE participant_id = rec.participant_id) AS b
                ON a.target_territories_included = b.target_territories_included AND a.pro_id != b.pro_id;
        COMMIT;
        INSERT INTO stg_last_conflict_check
        SELECT
            a.mandate_dim_id, b.target_territories, a.participant_id, a.pro_id,
            /* trim(regexp_substr(target_territories_included, '[^,]+', 1, levels.column_value)) as target_territories_included, */
            TRIM(aws_oracle_ext.regexp_substr((b.performance_types::TEXT)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels2.column_value::NUMERIC)::INTEGER)::NUMERIC)) AS performance_types, TRIM(aws_oracle_ext.regexp_substr((b.conditional_aspects::TEXT)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels3.column_value::NUMERIC)::INTEGER)::NUMERIC)) AS conditional_aspects, TRIM(aws_oracle_ext.regexp_substr((b.channels::TEXT)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels4.column_value::NUMERIC)::INTEGER)::NUMERIC)) AS channels, aws_oracle_ext.SYSDATE() AS dw_created_date
            FROM dim_mandate AS a, dim_mandate_detail AS b, stg_conflicts AS c,
            /* table(cast(multiset(select level from dual connect by level <= length(regexp_replace(target_territories_included, '[^,]+')) + 1) as sys.OdciNumberList)) levels */
            /* Transformer error occurred */,
            /* Transformer error occurred */,
            /* Transformer error occurred */
            /* where mandate_dim_id = 1074 */
            WHERE a.mandate_dim_id = b.mandate_dim_id AND b.mandate_dim_id = c.mandate_dim_id;
        COMMIT;
        INSERT INTO stg_final_mandate_conflicts
        SELECT
            y.mandate_dim_id, y.participant_id, y.pro_id, y.performance_types, y.conditional_aspects, y.channels, z.target_territories_included
            FROM stg_last_conflict_check AS y
            JOIN stg_conflicts AS z
                ON y.mandate_dim_id = z.mandate_dim_id;
        COMMIT;
        SELECT
            STRING_AGG(mandate_dim_id, ','::TEXT ORDER BY mandate_dim_id) AS conflict_mandate_dim_id
            INTO STRICT v_conflict_mandate_dim_id
            FROM (SELECT DISTINCT
                a.mandate_dim_id
                FROM stg_final_mandate_conflicts AS a, stg_final_mandate_conflicts AS b
                WHERE a.performance_types = b.performance_types AND a.conditional_aspects = b.conditional_aspects AND a.channels = b.channels AND a.target_territories_included = b.target_territories_included AND a.pro_id != b.pro_id AND b.mandate_dim_id = rec.mandate_dim_id
            EXCEPT
            SELECT
                rec.mandate_dim_id) AS var_sbq;

        IF v_conflict_mandate_dim_id IS NULL THEN
            v_conflict_mandate_dim_id := '1000';
        END IF;
        UPDATE dim_mandate_detail
        SET conflict_mandate_dim_id = v_conflict_mandate_dim_id
            WHERE mandate_dim_id = rec.mandate_dim_id;
        COMMIT;
    END LOOP /* end of conflicts loop */;
    UPDATE dim_mandate_detail
    SET conflict_mandate_dim_id = '1000'
        WHERE conflict_mandate_dim_id IS NULL;
    COMMIT;
    EXECUTE 'TRUNCATE TABLE dim_mandate_conflicts';
    /* inserts into dim_mandate_conflicts */
    INSERT INTO dim_mandate_conflicts
    SELECT
        mandate_dim_id, aws_oracle_ext.TO_NUMBER(TRIM(aws_oracle_ext.regexp_substr((conflict_mandate_dim_id::TEXT)::TEXT, ('[^,]+')::TEXT, (1)::NUMERIC, (TRUNC(levels.column_value::NUMERIC)::INTEGER)::NUMERIC))) AS conflict_mandate_dim_id, aws_oracle_ext.SYSDATE()
        FROM sx_edw_dim.dim_mandate_detail,
        /* Transformer error occurred */
        WHERE conflict_mandate_dim_id != '1000'::NUMERIC;
    COMMIT;
    /* ------------Mandate upgrade / Downgrade Logic----------------------- */
    /* column name is mandate_coverage_status */
    /* 0 = nochange, 1 = Downgrade, 2 = upgrade */
    FOR rec IN up_down_cur LOOP
        BEGIN
            /* This Begin/End added to handle not_data_found exceptions */
            v_country_length := 0;
            /* Grab the end date for the next newest record (if no_data_found then it is a new record) */
            SELECT
                MAX(date_to)
                INTO STRICT v_max_date
                FROM dim_mandate AS a
                WHERE a.status IN ('A', 'H') AND a.date_to < rec.date_to AND a.participant_id = rec.participant_id AND a.pro_id = rec.pro_id;
            /* rarely get more than one value here, changed query to handle it */
            /*
            [9996 - Severity CRITICAL - Transformer error occurred. Please submit report to developers.]
            select /*+ PARALLEL(8) */ distinct length(target_territories_included) country_length
            into v_country_length
            from (
            select /*+ PARALLEL(8) */ rownum rank, a.*,b.*
            from dim_mandate a join dim_mandate_detail b
            on a.mandate_dim_id = b.mandate_dim_id
            where a.status in ('A','H')
            --and exists (select /*+ PARALLEL(8) */ 1 from dim_mandate z where z.mandate_id = a.mandate_id and z.date_to > sysdate and z.status in ('A','H'))
            and a.participant_id = rec.participant_id
            and a.pro_id = rec.pro_id
            and a.date_to = v_max_date
            order by updated_on desc)
            where rownum = 1
            */
            /* Evaluate mandate change */
            IF
            /* no change */
            v_country_length = rec.country_length THEN
                UPDATE dim_mandate_detail
                SET mandate_coverage_status = 0
                    WHERE mandate_dim_id = rec.mandate_dim_id;
                COMMIT;
            /* downgrade */
            ELSIF v_country_length > rec.country_length THEN
                UPDATE dim_mandate_detail
                SET mandate_coverage_status = 1
                    WHERE mandate_dim_id = rec.mandate_dim_id;
                COMMIT;
            /* upgrade */
            ELSIF v_country_length < rec.country_length THEN
                UPDATE dim_mandate_detail
                SET mandate_coverage_status = 2
                    WHERE mandate_dim_id = rec.mandate_dim_id;
                COMMIT;
            END IF /* end of change evaluation */;
            EXCEPTION
                WHEN no_data_found THEN
                    UPDATE dim_mandate_detail
                    SET mandate_coverage_status = 0
                        WHERE mandate_dim_id = rec.mandate_dim_id;
                    COMMIT;
        END; /* This Begin/End added to handle not_data_found exceptions */
    END LOOP /* end of upgrade/downgrade loop */;
    /* set the remaining rows to 0 */
    UPDATE dim_mandate_detail
    SET mandate_coverage_status = 0
        WHERE mandate_coverage_status IS NULL;
    COMMIT;
END;
$BODY$
LANGUAGE plpgsql;



